@@ -1,5 +1,5 @@
 #
-#  Copyright (C) 2007, 2014, 2015, 2016, 2019, 2020
+#  Copyright (C) 2007, 2014, 2015, 2016, 2019, 2020, 2021
 #     Smithsonian Astrophysical Observatory
 #
 #
@@ -57,6 +57,7 @@ class Formatter(logging.Formatter):
             msg = record.msg
         return msg
 
+
 log = logging.getLogger('sherpa')
 handler = logging.StreamHandler(sys.stdout)
 handler.setFormatter(Formatter())
@@ -68,7 +69,6 @@ dbg = log.debug
 del Formatter, log, handler
 
 
-
 DEFAULT_CITATION = """Please review the Zenodo Sherpa page at
        https://doi.org/10.5281/zenodo.593753
 to identify the latest release of Sherpa. The Zenodo page
@@ -100,8 +100,8 @@ archivePrefix = {arXiv},
 
 @INPROCEEDINGS{2007ASPC..376..543D,
    author = {{Doe}, S. and {Nguyen}, D. and {Stawarz}, C. and {Refsdal}, B. and
-	{Siemiginowska}, A. and {Burke}, D. and {Evans}, I. and {Evans}, J. and
-	{McDowell}, J. and {Houck}, J. and {Nowak}, M.},
+        {Siemiginowska}, A. and {Burke}, D. and {Evans}, I. and {Evans}, J. and
+        {McDowell}, J. and {Houck}, J. and {Nowak}, M.},
     title = "{Developing Sherpa with Python}",
 booktitle = {Astronomical Data Analysis Software and Systems XVI},
      year = 2007,
@@ -164,30 +164,16 @@ def _get_citation_hardcoded(version):
     citation : str or None
         Citation information if known, otherwise None.
 
+    Notes
+    -----
+    The entries can be created with the script:
+    scripts/make_zenodo_release.py
+
     """
 
     def todate(year, mnum, dnum):
         return datetime.datetime(year, mnum, dnum)
 
-    db = 'DougBurke'
-    ol = 'Omar Laurino'
-    dn = 'dtnguyen2'
-    ta = 'Tom Aldcroft'
-    an = 'Aneta Siemiginowska'
-
-    jb = 'Jamie Budynkiewicz'
-    cd = 'Christoph Deil'
-    bs = 'Brigitta Sipocz'
-
-    wm = 'wmclaugh'
-
-    kl = 'Katrin Leinweber'
-
-    mt = 'Marie-Terrell'
-    bs2 = 'Brigitta Sipőcz'
-    hm = 'Hans Moritz Günther'
-    td = 'Todd'
-
     cite = {}
 
     def add(version, **kwargs):
@@ -195,58 +181,104 @@ def _get_citation_hardcoded(version):
         cite[version] = dict(**kwargs)
         cite[version]['version'] = version
 
-    add(version='4.8.0', title='sherpa: Sherpa 4.8.0',
-        date=todate(2016, 1, 27),
-        authors=[db, ol, dn, ta, an],
-        idval='45243')
-    add(version='4.8.1', title='sherpa: Sherpa 4.8.1',
-        date=todate(2016, 4, 15),
-        authors=[db, ol, dn, ta, jb, an, cd, bs],
-        idval='49832')
-    add(version='4.8.2', title='sherpa: Sherpa 4.8.2',
-        date=todate(2016, 9, 23),
-        authors=[db, ol, dn, jb, ta, an, cd, wm, bs],
-        idval='154744')
+    add(version='4.13.0', title='sherpa/sherpa: Sherpa 4.13.0',
+        date=todate(2021, 1, 8),
+        authors=['Doug Burke', 'Omar Laurino', 'wmclaugh', 'dtnguyen2',
+                 'Marie-Terrell', 'Hans Moritz Günther', 'Aneta Siemiginowska',
+                 'Jamie Budynkiewicz', 'Tom Aldcroft', 'Christoph Deil',
+                 'Brigitta Sipőcz', 'Johannes Buchner', 'Iva Laginja',
+                 'Katrin Leinweber', 'nplee', 'Todd'],
+        idval='4428938')
 
-    add(version='4.9.0', title='sherpa/sherpa: Sherpa 4.9.0',
-        date=todate(2017, 1, 27),
-        authors=[db, ol, dn, jb, ta, an, wm, cd, bs],
-        idval='260416')
-    add(version='4.9.1', title='sherpa/sherpa: Sherpa 4.9.1',
-        date=todate(2017, 8, 3),
-        authors=[db, ol, dn, jb, ta, an, wm, cd, bs],
-        idval='838686')
-
-    add(version='4.10.0', title='sherpa/sherpa: Sherpa 4.10.0',
-        date=todate(2018, 5, 1),
-        authors=[db, ol, dn, jb, ta, an, wm, cd, bs],
-        idval='1245678')
-    add(version='4.10.1', title='sherpa/sherpa: Sherpa 4.10.1',
-        date=todate(2018, 10, 16),
-        authors=[db, ol, dn, jb, ta, an, wm, bs, cd, kl],
-        idval='1463962')
-    add(version='4.10.2', title='sherpa/sherpa: Sherpa 4.10.2',
-        date=todate(2018, 12, 14),
-        authors=[db, ol, dn, jb, ta, an, cd, wm, bs, kl],
-        idval='2275738')
-
-    add(version='4.11.0', title='sherpa/sherpa: Sherpa 4.11.0',
-        date=todate(2019, 2, 20),
-        authors=[db, ol, dn, jb, ta, an, cd, wm, bs, kl],
-        idval='2573885')
-    add(version='4.11.1', title='sherpa/sherpa: Sherpa 4.11.1',
-        date=todate(2019, 8, 1),
-        authors=[db, ol, dn, jb, ta, an, cd, wm, bs, kl],
-        idval='3358134')
-
-    add(version='4.12.0', title='sherpa/sherpa: Sherpa 4.12.0',
-        date=todate(2020, 1, 30),
-        authors=[db, ol, dn, wm, jb, ta, an, cd, mt, bs2, hm, td, kl],
-        idval='3631574')
+    add(version='4.12.2', title='sherpa/sherpa: Sherpa 4.12.2',
+        date=todate(2020, 10, 27),
+        authors=['Doug Burke', 'Omar Laurino', 'wmclaugh', 'dtnguyen2',
+                 'Hans Moritz Günther', 'Marie-Terrell', 'Aneta Siemiginowska',
+                 'Jamie Budynkiewicz', 'Tom Aldcroft', 'Christoph Deil',
+                 'Brigitta Sipőcz', 'Johannes Buchner', 'Iva Laginja',
+                 'Katrin Leinweber', 'nplee', 'Todd'],
+        idval='4141888')
     add(version='4.12.1', title='sherpa/sherpa: Sherpa 4.12.1',
         date=todate(2020, 7, 14),
-        authors=[db, ol, wm, dn, mt, hm, jb, an, ta, cd, bs2, kl, td],
+        authors=['Doug Burke', 'Omar Laurino', 'wmclaugh', 'dtnguyen2',
+                 'Marie-Terrell', 'Hans Moritz Günther', 'Jamie Budynkiewicz',
+                 'Aneta Siemiginowska', 'Tom Aldcroft', 'Christoph Deil',
+                 'Brigitta Sipőcz', 'Katrin Leinweber', 'Todd'],
         idval='3944985')
+    add(version='4.12.0', title='sherpa/sherpa: Sherpa 4.12.0',
+        date=todate(2020, 1, 30),
+        authors=['Doug Burke', 'Omar Laurino', 'dtnguyen2', 'wmclaugh',
+                 'Jamie Budynkiewicz', 'Tom Aldcroft', 'Aneta Siemiginowska',
+                 'Christoph Deil', 'Marie-Terrell', 'Brigitta Sipőcz',
+                 'Hans Moritz Günther', 'Todd', 'Katrin Leinweber'],
+        idval='3631574')
+
+    add(version='4.11.1', title='sherpa/sherpa: Sherpa 4.11.1',
+        date=todate(2019, 8, 1),
+        authors=['Doug Burke', 'Omar Laurino', 'dtnguyen2',
+                 'Jamie Budynkiewicz', 'Tom Aldcroft', 'Aneta Siemiginowska',
+                 'Christoph Deil', 'wmclaugh', 'Brigitta Sipocz',
+                 'Katrin Leinweber'],
+        idval='3358134')
+    add(version='4.11.0', title='sherpa/sherpa: Sherpa 4.11.0',
+        date=todate(2019, 2, 20),
+        authors=['Doug Burke', 'Omar Laurino', 'dtnguyen2',
+                 'Jamie Budynkiewicz', 'Tom Aldcroft', 'Aneta Siemiginowska',
+                 'Christoph Deil', 'wmclaugh', 'Brigitta Sipocz',
+                 'Katrin Leinweber'],
+        idval='2573885')
+
+    add(version='4.10.2', title='sherpa/sherpa: Sherpa 4.10.2',
+        date=todate(2018, 12, 14),
+        authors=['Doug Burke', 'Omar Laurino', 'dtnguyen2',
+                 'Jamie Budynkiewicz', 'Tom Aldcroft', 'Aneta Siemiginowska',
+                 'Christoph Deil', 'wmclaugh', 'Brigitta Sipocz',
+                 'Katrin Leinweber'],
+        idval='2275738')
+    add(version='4.10.1', title='sherpa/sherpa: Sherpa 4.10.1',
+        date=todate(2018, 10, 16),
+        authors=['Doug Burke', 'Omar Laurino', 'dtnguyen2',
+                 'Jamie Budynkiewicz', 'Tom Aldcroft', 'Aneta Siemiginowska',
+                 'wmclaugh', 'Brigitta Sipocz', 'Christoph Deil',
+                 'Katrin Leinweber'],
+        idval='1463962')
+    add(version='4.10.0', title='sherpa/sherpa: Sherpa 4.10.0',
+        date=todate(2018, 5, 11),
+        authors=['Doug Burke', 'Omar Laurino', 'dtnguyen2',
+                 'Jamie Budynkiewicz', 'Tom Aldcroft', 'Aneta Siemiginowska',
+                 'wmclaugh', 'Christoph Deil', 'Brigitta Sipocz'],
+        idval='1245678')
+
+    add(version='4.9.1', title='sherpa/sherpa: Sherpa 4.9.1',
+        date=todate(2017, 8, 3),
+        authors=['Doug Burke', 'Omar Laurino', 'dtnguyen2',
+                 'Jamie Budynkiewicz', 'Tom Aldcroft', 'Aneta Siemiginowska',
+                 'wmclaugh', 'Christoph Deil', 'Brigitta Sipocz'],
+        idval='838686')
+    add(version='4.9.0', title='sherpa/sherpa: Sherpa 4.9.0',
+        date=todate(2017, 1, 27),
+        authors=['Doug Burke', 'Omar Laurino', 'dtnguyen2',
+                 'Jamie Budynkiewicz', 'Tom Aldcroft', 'Aneta Siemiginowska',
+                 'wmclaugh', 'Christoph Deil', 'Brigitta Sipocz'],
+        idval='260416')
+
+    add(version='4.8.2', title='sherpa/sherpa: Sherpa 4.8.2',
+        date=todate(2016, 9, 23),
+        authors=['Doug Burke', 'Omar Laurino', 'dtnguyen2',
+                 'Jamie Budynkiewicz', 'Tom Aldcroft', 'Aneta Siemiginowska',
+                 'Christoph Deil', 'wmclaugh', 'Brigitta Sipocz'],
+        idval='154744')
+    add(version='4.8.1', title='sherpa: Sherpa 4.8.1',
+        date=todate(2016, 4, 15),
+        authors=['Doug Burke', 'Omar Laurino', 'dtnguyen2', 'Tom Aldcroft',
+                 'Jamie Budynkiewicz', 'Aneta Siemiginowska', 'Christoph Deil',
+                 'Brigitta Sipocz'],
+        idval='49832')
+    add(version='4.8.0', title='sherpa: Sherpa 4.8.0',
+        date=todate(2016, 1, 27),
+        authors=['Doug Burke', 'Omar Laurino', 'dtnguyen2', 'Tom Aldcroft',
+                 'Aneta Siemiginowska'],
+        idval='45243')
 
     kwargs = cite.get(version, None)
     if kwargs is None:
@@ -379,7 +411,7 @@ def _get_citation_zenodo_failure(failed):
         The failure message.
 
     """
-    out = 'There was a problem retireving the data from Zenodo:\n'
+    out = 'There was a problem retrieving the data from Zenodo:\n'
     out += failed
     out += '\n\n'
     out += DEFAULT_CITATION
@@ -412,6 +444,23 @@ def _get_citation_zenodo_latest():
     return out['success']
 
 
+def _zenodo_missing(version):
+    """
+
+    Parameters
+    ----------
+    version : str
+        The version we are looking for.
+
+    Returns
+    -------
+    message : str
+        The "unable to find" message.
+    """
+
+    return 'Zenodo has no information for version {}.'.format(version)
+
+
 def _parse_zenodo_data(jsdata, version):
     """Extract data for the given version from a Zenodo query.
 
@@ -451,33 +500,104 @@ def _parse_zenodo_data(jsdata, version):
 
     if data is None:
         dbg('Version {} not found'.format(version))
-        return {'failed': "Zenodo has no information for version {}.".format(version)}
+        return {'failed': _zenodo_missing(version)}
 
     return {'success': data}
 
 
+def _download_zenodo_data(version):
+    """Query Zenodo for the specific release.
+
+    We have to deal with pagination in the Zenodo response.
+
+    Parameters
+    ----------
+    version : str
+        The release number (e.g. '4.12.2').
+
+    Returns
+    -------
+    response : dict
+        The data in JSON as 'success' or an error message as 'failed'.
+
+    """
+
+    # We could set the size parameter to something very large, to
+    # get all responses with one call, but instead we use pagination.
+    # Zenodo helpfully provides a links/next record with the
+    # next URL, but it seems to be missing the all_versions=True
+    # option, which makes it less-than-useful, hence the addition
+    # of it below. The alternative would be to manually track the
+    # page counter and add '&page=n' to the call.
+    #
+    url = 'https://zenodo.org/api/records/?q=conceptrecid:"593753"&all_versions=True&sort=mostrecent'
+    missing = _zenodo_missing(version)
+
+    while True:
+        dbg("Zenodo query: {}".format(url))
+        jsdata = _download_json(url)
+
+        # If the query fails then we error out
+        #
+        if 'failed' in jsdata:
+            return jsdata
+
+        jsdata = jsdata['success']
+        data = _parse_zenodo_data(jsdata, version)
+        if 'success' in data:
+            return data
+
+        # There are two failures we care about:
+        #  - we can parse the information but have not been able to
+        #    find the version
+        #  - any other reason
+        #
+        # If the former then we look for the links/next entry to
+        # look at the next page of the response. If it doesn't
+        # exist we assume we are on the last page and so can error
+        # out.
+        #
+        # If the latter then we error out rather than trying anything
+        # else.
+        #
+        if data['failed'] != missing:
+            return data
+
+        try:
+            url = jsdata['links']['next']
+
+            # Add in the necessary all_versions tag: see
+            # https://github.com/zenodo/zenodo/issues/1662
+            #
+            if 'all_versions=True' not in url:
+                url += '&all_versions=True'
+
+        except KeyError:
+            # There is no links/next field so assume we've checked all
+            # pages
+            return data
+
+
 def _get_citation_zenodo_version(version):
     """Query Zenodo for the specific release.
 
     As this has to return all Sherpa records it is slow.
 
+    Parameters
+    ----------
+    version : str
+        The release number (e.g. '4.12.2').
+
     Returns
     -------
     citation : str
         Citation information.
     """
 
-    # Is there a better way to do this?
-    #
-    url = 'https://zenodo.org/api/records/?q=conceptrecid:"593753"&all_versions=True'
-    jsdata = _download_json(url)
+    jsdata = _download_zenodo_data(version)
     if 'failed' in jsdata:
         return _get_citation_zenodo_failure(jsdata['failed'])
 
-    data = _parse_zenodo_data(jsdata['success'], version)
-    if 'failed' in data:
-        return _get_citation_zenodo_failure(data['failed'])
-
     out = _make_zenodo_citation(jsdata['success'], latest=False)
     if 'failed' in out:
         return _get_citation_zenodo_failure(out['failed'])
@@ -621,12 +741,12 @@ def get_config():
 
     # If NOSHERPARC is set, read in system config file
     # ignore any user config file
-    if (('NOSHERPARC' in os.environ) == True):
+    if 'NOSHERPARC' in os.environ:
         return os.path.join(os.path.dirname(__file__), filename)
 
     # If SHERPARC is set, read in config file from there,
     # and ignore default location
-    if (('SHERPARC' in os.environ) == True):
+    if 'SHERPARC' in os.environ:
         config = os.environ.get('SHERPARC')
         if os.path.isfile(config):
             return config
@@ -708,7 +828,7 @@ def _install_test_deps():
     def install(package_name):
         try:
             subprocess.call([sys.executable, '-m', 'pip', 'install', package_name],
-                               stdout=sys.stdout, stderr=sys.stderr)
+                            stdout=sys.stdout, stderr=sys.stderr)
         except:
             print("""Cannot import pip or install packages with it.
             You need pytest, and possibly pytest-cov, in order to run the tests.
@@ -744,9 +864,15 @@ def _install_test_deps():
 
 
 def clitest():
+    """The sherpa_test endpoint."""
+
     plugins = _install_test_deps()
     import pytest
     import os
-    sherpa_dir = os.path.dirname(__file__)
-    errno = pytest.main([sherpa_dir, '-rs'], plugins=plugins)  # passing the plugins that have been installed "now".
+
+    # Add in command-line arguments to allow configuring the Sherpa tests
+    args = [os.path.dirname(__file__), '-rs'] + sys.argv[1:]
+
+    # passing the plugins that have been installed "now".
+    errno = pytest.main(args, plugins=plugins)
     sys.exit(errno)
