@@ -1,5 +1,5 @@
 #
-#  Copyright (C) 2007, 2020  Smithsonian Astrophysical Observatory
+#  Copyright (C) 2007, 2020, 2021  Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -22,9 +22,7 @@
 """
 
 from collections import defaultdict
-from functools import reduce
 import logging
-import operator
 
 import numpy as np
 
