@@ -1,6 +1,6 @@
 #
-#  Copyright (C) 2009, 2015, 2016, 2019, 2020
-#       Smithsonian Astrophysical Observatory
+#  Copyright (C) 2009, 2015, 2016, 2019, 2020, 2021
+#  Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -38,6 +38,7 @@ from sherpa.sim import NormalParameterSampleFromScaleMatrix, \
     NormalParameterSampleFromScaleVector
 from sherpa.models.model import SimulFitModel
 
+info = logging.getLogger(__name__).info
 
 __all__ = ['calc_flux', 'sample_flux', 'calc_sample_flux']
 
@@ -441,7 +442,7 @@ def sample_flux(fit, data, src,
 
     See Also
     --------
-    calc_flux
+    calc_flux, calc_sample_flux
 
     Notes
     -----
@@ -526,94 +527,142 @@ def sample_flux(fit, data, src,
     return numpy.concatenate((vals, numpy.expand_dims(clipped, 1)), axis=1)
 
 
-def calc_sample_flux(id, lo, hi, session, fit, data, samples, modelcomponent,
+def calc_sample_flux(lo, hi, fit, data, samples, modelcomponent,
                      confidence):
+    """Given a set of parameter samples, estimate the flux distribution.
 
-    def simulated_pars_within_ranges(mysamples, mysoftmins, mysoftmaxs):
-        num = len(mysoftmins)
-        for ii in range(num):
-            ii1 = ii + 1
-            tmp = (mysamples[:, ii1] > mysoftmins[ii])
-            selpmasym = mysamples[tmp]
-            tmp = (selpmasym[:, ii1] < mysoftmaxs[ii])
-            mysamples = selpmasym[tmp]
-        return mysamples
+    This is similar to sample_flux but returns values for both the full
+    and component models (which can be the same). The set of parameter
+    samples is searched to remove rows where the parameters lie outside
+    the soft limits, and then this set of rows is used to calculate the
+    flux of the modelcomponent expression (the flux for the full model
+    is taken from the samples argument). The resulting set of fluxes is
+    used to calculate the median and the confidence range.
 
-    def print_sample_result(title, arg):
+    .. versionchanged:: 4.13.1
+       The clipping now includes parameters at the soft limits;
+       previously they were excluded which could cause problems with
+       parameters for which we can only calculate an upper limit. The
+       id and session arguments have been removed. The statistic value
+       is now returned for each row, even those that were excluded
+       from the flux calculation.
 
-        print('%s = %g, + %g, - %g' % (title, arg[0], arg[1] - arg[0],
-                                       arg[0] - arg[2]))
+
+    Parameters
+    ----------
+    lo : number or None, optional
+        The lower edge of the dataspace range for the flux calculation.
+        If None then the lower edge of the data grid is used.
+    hi : number or None, optional
+        The upper edge of the dataspace range for the flux calculation.
+        If None then the upper edge of the data grid is used.
+    fit : sherpa.fit.Fit instance
+        The fit object. The src parameter is assumed to be a subset of
+        the fit.model expression (to allow for calculating the flux of
+        a model component), and the fit.data object matches the data
+        object. The fit.model argument is expected to include instrumental
+        models (for PHA data sets). These objects can represent
+        simultaneous fits (e.g. sherpa.data.DataSimulFit and
+        sherpa.models.model.SimulFitModel).
+    data : sherpa.data.Data subclass
+        The data object to use. This is not a DataSimulFit instance.
+    samples
+        The output of sample_flux for the data (assumed to have been
+        created with method=calc_energy_flux and clip='hard').
+    modelcomponent : sherpa.models.Arithmetic instance
+        The source model (without instrument response for PHA data) that
+        is used for calculating the flux. This is not a SimulFitModel
+        instance.
+    confidence : number, optional
+       The confidence level for the upper and lower values, as a
+       percentage (0 to 100). A value of 68.3 will return the
+       one-sigma range.
+
+    Returns
+    -------
+    (fullflux, cptflux, vals)
+        The fullflux and cptflux arrays contain the results for the
+        full source model and the flux of the `modelcomponent`
+        argument (they can be the same). They have three elements and
+        give the median value, the value containing 100 - confidence/2
+        of the data, and the fraction containing confidence/2 of the
+        flux distribution. For the default confidence argument of 68
+        this means the last two give the one-sigma upper and lower
+        bounds. The vals array has a shape of ``(num+1, N+3)``, where
+        ``N`` is the number of free parameters and num is the `num`
+        parameter. The rows of this array contain the flux value for
+        the iteration (for the full source model), the parameter
+        values, a flag indicating whether any parameter in that row
+        was clipped to the hard-limits of the parameter, and the
+        statistic value for this set of parameters.
+
+    See Also
+    --------
+    calc_flux, calc_sample_flux
+
+    Notes
+    -----
+    This function is in the process of being documented and
+    re-written, as it contains a lot of redundancy and the semantics
+    of some of the options are unclear.
+
+    The summary output displayed by this routine - giving the median
+    and confidence ranges - is controlled by the standard Sherpa
+    logging instance, and can be hidden by changing the logging to a
+    level greater than "INFO" (e.g. with
+    `sherpa.utils.logging.SherpaVerbosity`).
+
+    """
+
+    thawedpars = fit.model.thawedpars
+
+    # Check the number of free parameters agrees with the samples argument,
+    # noting that each row in samples is <flux> + <free pars> + <clip>. This is
+    # a requirement for calling this routine, so is just handled as an
+    # assert (as this is not intended for a user-level routine)
     #
-    # For later restoration
-    #
-    orig_model = session.get_model(id)
-    orig_model_vals = fit.model.thawedpars
+    nthawed = len(thawedpars)
+    npars = samples.shape[1] - 2
+    assert nthawed == npars, (nthawed, npars)
 
-    orig_source = session.get_source(id)
-    orig_source_vals = orig_source.thawedpars
+    # Identify any row where a parameter lies outside the min/max range
+    softmins = fit.model.thawedparmins
+    softmaxs = fit.model.thawedparmaxes
 
+    # We have to use columns 1 to n-1 of samples
+    valid = numpy.ones(samples.shape[0], dtype=numpy.bool)
+    for col, pmin, pmax in zip(samples.T[1:-1], softmins, softmaxs):
+        valid &= (col >= pmin) & (col <= pmax)
+
+    nrows = samples.shape[0]
+    oflx = samples[:, 0]       # observed/absorbed flux
+    iflx = numpy.zeros(nrows)  # intrinsic/unabsorbed flux
+
+    mystat = numpy.zeros((nrows, 1), dtype=samples.dtype)
     try:
+        for nn in range(nrows):
+            # Need to extract the subset that contains the parameters
+            fit.model.thawedpars = samples[nn, 1:-1]
+            if valid[nn]:
+                iflx[nn] = calc_energy_flux(data, modelcomponent, lo=lo, hi=hi)
 
-        softmins = fit.model.thawedparmins
-        softmaxs = fit.model.thawedparmaxes
-        mysim = simulated_pars_within_ranges(samples, softmins, softmaxs)
-
-        size = len(mysim[:, 0])
-        oflx = numpy.zeros(size)  # observed/absorbed flux
-        iflx = numpy.zeros(size)  # intrinsic/unabsorbed flux
-        thawedpars = [par for par in fit.model.pars if not par.frozen]
-
-        logger = logging.getLogger("sherpa")
-        orig_log_level = logger.level
-
-        mystat = []
-        for nn in range(size):
-            logger.setLevel(logging.ERROR)
-            session.set_source(id, orig_source)
-            logger.setLevel(orig_log_level)
-            oflx[nn] = mysim[nn, 0]
-            for ii in range(len(thawedpars)):
-                val = mysim[nn, ii + 1]
-                session.set_par(thawedpars[ii].fullname, val)
-            session.set_source(id, modelcomponent)
-            iflx[nn] = session.calc_energy_flux(lo, hi)
-            #####################################
-            session.set_full_model(id, orig_model)
-            mystat.append(session.calc_stat())
-            #####################################
-        oflxiflx = [oflx, iflx]
-
-        myconfidence = (1.0 - confidence / 100.0) / 2.0
-        result = []
-
-        for x in oflxiflx:
-            sf = numpy.sort(x)
-            median = numpy.median(sf)
-            upconfidence_index = int((1.0 - myconfidence) * size - 1)
-            loconfidence_index = int(myconfidence * size - 1)
-            upconfidence = sf[upconfidence_index]
-            loconfidence = sf[loconfidence_index]
-            result.append(numpy.array([median, upconfidence, loconfidence]))
-
-        print_sample_result('original model flux', result[0])
-        print_sample_result('model component flux', result[1])
-
-        sampletmp = numpy.zeros((samples.shape[0], 1), dtype=samples.dtype)
-        samples = numpy.concatenate((samples, sampletmp), axis=1)
-
-        for index in range(size):
-            samples[index][-1] = mystat[index]
-
-        #samples = numpy.delete( samples, (size), axis=0 )
-        result.append(samples)
-
-        return result
+            mystat[nn, 0] = fit.calc_stat()
 
     finally:
+        fit.model.thawedpars = thawedpars
 
-        session.set_full_model(id, orig_model)
-        fit.model.thawedpars = orig_model_vals
+    hwidth = confidence / 2
+    result = []
+    for flx in [oflx, iflx]:
+        result.append(numpy.percentile(flx[valid],
+                                       [50, 50 + hwidth, 50 - hwidth]))
 
-        logger.setLevel(logging.ERROR)
-        session.set_source(id, orig_source)
-        logger.setLevel(orig_log_level)
+    for lbl, arg in zip(['original model', 'model component'], result):
+        med, usig, lsig = arg
+        msg = '{} flux = {:g}, + {:g}, - {:g}'.format(lbl, med, usig - med, med - lsig)
+        info(msg)
+
+    samples = numpy.concatenate((samples, mystat), axis=1)
+    result.append(samples)
+
+    return result
