@@ -1,5 +1,5 @@
 #
-#  Copyright (C) 2010, 2015-2018, 2019, 2020
+#  Copyright (C) 2010, 2015-2018, 2019, 2020, 2021
 #     Smithsonian Astrophysical Observator
 #
 #
@@ -38,8 +38,10 @@ References
 
 """
 
-import numpy
 import os
+
+import numpy
+
 import sherpa
 from sherpa.utils.err import InstrumentErr, DataErr, PSFErr
 from sherpa.models.model import ArithmeticModel, CompositeModel, Model
@@ -55,7 +57,7 @@ from sherpa.astro.utils import compile_energy_grid
 WCS = None
 try:
     from sherpa.astro.io.wcs import WCS
-except:
+except ImportError:
     WCS = None
 
 _tol = numpy.finfo(numpy.float32).eps
@@ -1140,7 +1142,6 @@ class PileupResponse1D(NoNewAttributesAfterInit):
 
     """
 
-
     def __init__(self, pha, pileup_model):
         self.pha = pha
         self.pileup_model = pileup_model
