@@ -1,6 +1,6 @@
 #
-#  Copyright (C) 2011, 2015, 2016, 2017, 2018, 2019, 2020
-#    Smithsonian Astrophysical Observatory
+#  Copyright (C) 2011, 2015, 2016, 2017, 2018, 2019, 2020, 2021
+#  Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -37,22 +37,22 @@ References
 
 """
 
+import logging
+import os
+import warnings
+
 import numpy
+from numpy.compat import basestring
 
 from astropy.io import fits
 from astropy.io.fits.column import _VLF
 
-from numpy.compat import basestring
-
-import os
-
 from sherpa.utils.err import IOErr
 from sherpa.utils import SherpaInt, SherpaUInt, SherpaFloat
 import sherpa.utils
 from sherpa.io import get_ascii_data, write_arrays
 from sherpa.astro.io.meta import Meta
 
-import logging
 warning = logging.getLogger(__name__).warning
 error = logging.getLogger(__name__).error
 
@@ -60,7 +60,7 @@ transformstatus = False
 try:
     from sherpa.astro.io.wcs import WCS
     transformstatus = True
-except:
+except ImportError:
     warning('failed to import WCS module; WCS routines will not be ' +
             'available')
 
@@ -69,9 +69,9 @@ __all__ = ('get_table_data', 'get_image_data', 'get_arf_data', 'get_rmf_data',
            'get_column_data', 'get_ascii_data')
 
 
-def _has_hdu(hdulist, id):
+def _has_hdu(hdulist, name):
     try:
-        hdulist[id]
+        hdulist[name]
     except (KeyError, IndexError):
         return False
     return True
@@ -108,7 +108,7 @@ def _get_meta_data(hdu):
         val = hdu.header[key]
 
         # empty numpy strings are not recognized by load pickle!
-        if type(val) is numpy.str_ and val == '':
+        if isinstance(val, numpy.str_) and val == '':
             val = ''
 
         meta[key] = val
@@ -286,7 +286,20 @@ def open_fits(filename):
        function.
     """
     fname = _infer_and_check_filename(filename)
-    return fits.open(fname)
+
+    # With AstroPy v4.2.1 we start to see warnings when this call
+    # fails (i.e. when the argument is not a FITS file).  This leads
+    # to spurious messages to the user, so we just remove all
+    # warnings. This may need tweaking at some point: ideally we would
+    # only want to hide the warnings when it is not a FITS file but
+    # show them when it is a FITS file.
+    #
+    # Note that this is not thread safe.
+    with warnings.catch_warnings():
+        warnings.filterwarnings('ignore', module='astropy.io.fits')
+        out = fits.open(fname)
+
+    return out
 
 
 def read_table_blocks(arg, make_copy=False):
@@ -685,7 +698,7 @@ def get_rmf_data(arg, make_copy=False):
             # QUS: should this actually be an error, rather than just
             #      something that is logged to screen?
             error("Failed to locate TLMIN keyword for F_CHAN" +
-                  " column in RMF file '%s'; "  % filename +
+                  " column in RMF file '%s'; " % filename +
                   'Update the offset value in the RMF data set to' +
                   ' appropriate TLMIN value prior to fitting')
 
@@ -736,7 +749,7 @@ def get_rmf_data(arg, make_copy=False):
         rowdata = []
         for mrow, ng, ncs in zip(matrix, n_grp, n_chan):
             # Need a RMF which ng>1 to test this with.
-            if ng == 1:
+            if numpy.isscalar(ncs):
                 ncs = [ncs]
 
             start = 0
@@ -749,9 +762,9 @@ def get_rmf_data(arg, make_copy=False):
                 # "perfect" RMFs may have mrow as a scalar
                 try:
                     rdata = mrow[start:end]
-                except IndexError:
+                except IndexError as ie:
                     if start != 0 or end != 1:
-                        raise IOErr('bad', 'format', 'MATRIX column formatting')
+                        raise IOErr('bad', 'format', 'MATRIX column formatting') from ie
 
                     rdata = [mrow]
 
@@ -819,8 +832,8 @@ def get_pha_data(arg, make_copy=False, use_background=False):
             data['exposure'] = _try_key(hdu, 'EXPOSURE', True, SherpaFloat)
             # data['poisserr'] = _try_key(hdu, 'POISSERR', True, bool)
             data['backfile'] = _try_key(hdu, 'BACKFILE')
-            data['arffile']  = _try_key(hdu, 'ANCRFILE')
-            data['rmffile']  = _try_key(hdu, 'RESPFILE')
+            data['arffile'] = _try_key(hdu, 'ANCRFILE')
+            data['rmffile'] = _try_key(hdu, 'RESPFILE')
 
             # Keywords or columns
             data['backscal'] = _try_col_or_key(hdu, 'BACKSCAL', fix_type=True)
@@ -877,8 +890,8 @@ def get_pha_data(arg, make_copy=False, use_background=False):
             exposure = _try_key(hdu, 'EXPOSURE', True, SherpaFloat)
             # poisserr = _try_key(hdu, 'POISSERR', True, bool)
             backfile = _try_key(hdu, 'BACKFILE')
-            arffile  = _try_key(hdu, 'ANCRFILE')
-            rmffile  = _try_key(hdu, 'RESPFILE')
+            arffile = _try_key(hdu, 'ANCRFILE')
+            rmffile = _try_key(hdu, 'RESPFILE')
 
             # Keywords or columns
             backscal = _try_vec_or_key(hdu, 'BACKSCAL', num, fix_type=True)
@@ -1084,7 +1097,7 @@ def set_image_data(filename, data, header, ascii=False, clobber=False,
         cdeltw = data['eqpos'].cdelt
         crpixw = data['eqpos'].crpix
         crvalw = data['eqpos'].crval
-        equin  = data['eqpos'].equinox
+        equin = data['eqpos'].equinox
 
     if data['sky'] is not None:
         cdeltp = data['sky'].cdelt
@@ -1106,7 +1119,7 @@ def set_image_data(filename, data, header, ascii=False, clobber=False,
         if data['eqpos'] is not None:
             # Simply the inverse of read transformations in get_image_data
             cdeltw = cdeltw * cdeltp
-            crpixw = ((crpixw - crvalp) / cdeltp + crpixp )
+            crpixw = ((crpixw - crvalp) / cdeltp + crpixp)
 
     if data['eqpos'] is not None:
         _add_keyword(hdrlist, 'MTYPE2', 'EQPOS   ')
@@ -1149,7 +1162,7 @@ def set_arrays(filename, args, fields=None, ascii=True, clobber=False):
     for arg in args:
         if not numpy.iterable(arg):
             raise IOErr('noarrayswrite')
-        elif len(arg) != size:
+        if len(arg) != size:
             raise IOErr('arraysnoteq')
 
     if fields is None:
