@@ -1,5 +1,6 @@
 #
-#  Copyright (C) 2010, 2015, 2016, 2019, 2020  Smithsonian Astrophysical Observatory
+#  Copyright (C) 2010, 2015, 2016, 2019, 2020, 2021
+#  Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -27,7 +28,7 @@ import numpy as np
 from numpy import iterable, array2string, asarray
 
 from sherpa.astro.data import DataPHA
-from sherpa.plot import DataPlot, ModelPlot, FitPlot, DelchiPlot, ResidPlot, \
+from sherpa.plot import FitPlot, DelchiPlot, ResidPlot, \
     RatioPlot, ChisqrPlot, HistogramPlot, backend, Histogram
 from sherpa.plot import ComponentSourcePlot as _ComponentSourcePlot
 import sherpa.plot
@@ -38,11 +39,13 @@ from sherpa.utils import parse_expr, dataspace1d, histogram1d, filter_bins, \
 
 warning = logging.getLogger(__name__).warning
 
-__all__ = ('DataPHAPlot', 'SourcePlot', 'ComponentModelPlot',
-           'ComponentSourcePlot', 'ARFPlot', 'BkgDataPlot',
-           'BkgFitPlot', 'BkgSourcePlot', 'BkgDelchiPlot', 'BkgResidPlot',
-           'BkgRatioPlot', 'BkgChisqrPlot',
-           'OrderPlot', 'ModelHistogram', 'BkgModelHistogram',
+__all__ = ('DataPHAPlot', 'ModelPHAHistogram', 'ModelHistogram',
+           'SourcePlot', 'ComponentModelPlot', 'ComponentSourcePlot',
+           'ARFPlot',
+           'BkgDataPlot', 'BkgModelPHAHistogram', 'BkgModelHistogram',
+           'BkgFitPlot', 'BkgDelchiPlot', 'BkgResidPlot', 'BkgRatioPlot',
+           'BkgChisqrPlot', 'BkgSourcePlot',
+           'OrderPlot',
            'FluxHistogram', 'EnergyFluxHistogram', 'PhotonFluxHistogram')
 
 
@@ -224,7 +227,7 @@ class SourcePlot(HistogramPlot):
 
     def __init__(self):
         self.units = None
-        self.mask  = None
+        self.mask = None
         HistogramPlot.__init__(self)
         self.title = 'Source'
 
@@ -242,7 +245,7 @@ class SourcePlot(HistogramPlot):
             self.units = "energy"
 
         self.xlabel = data.get_xlabel()
-        self.title  = 'Source Model of %s' % data.name
+        self.title = 'Source Model of %s' % data.name
         self.xlo, self.xhi = data._get_indep(filter=False)
         self.mask = filter_bins((lo,), (hi,), (self.xlo,))
         self.y = src(self.xlo, self.xhi)
@@ -615,8 +618,8 @@ class FluxHistogram(ModelHistogram):
     def prepare(self, fluxes, bins):
         """Define the histogram plot.
 
-        Parameter
-        ---------
+        Parameters
+        ----------
         fluxes : numpy array
             The data, stored in a niter by (npar + 2) matrix, where
             each row is an iteration, the first column is the flux for
