@@ -1,5 +1,5 @@
 #
-#  Copyright (C) 2012, 2015, 2016, 2017, 2018, 2019, 2020
+#  Copyright (C) 2012, 2015, 2016, 2017, 2018, 2019, 2020, 2021
 #     Smithsonian Astrophysical Observatory
 #
 #
@@ -21,29 +21,33 @@
 from collections import namedtuple
 import os
 import re
-import unittest
 import tempfile
 import logging
 
-import pytest
-
 import numpy
 from numpy.testing import assert_allclose
 
+import pytest
+
+from sherpa.astro.data import DataPHA
+from sherpa.astro.instrument import RMFModelPHA
+from sherpa.astro import ui
+from sherpa.data import Data1D
+from sherpa.utils.err import ArgumentErr, DataErr, IdentifierErr, StatErr
+from sherpa.utils.logging import SherpaVerbosity
 from sherpa.utils.testing import requires_data, requires_fits, \
     requires_group, requires_xspec
-from sherpa.utils.err import DataErr, IdentifierErr, StatErr
-from sherpa.astro import ui
-from sherpa.astro.instrument import RMFModelPHA
-from sherpa.data import Data1D
-from sherpa.astro.data import DataPHA
 
 logger = logging.getLogger("sherpa")
 
 
 @pytest.fixture(autouse=True)
 def hide_logging():
-    "hide INFO-level logging in all these tests"
+    """hide INFO-level logging in all these tests
+
+    This replicates the conftest hide_logging but applies it to
+    all tests.
+    """
 
     olevel = logger.getEffectiveLevel()
     logger.setLevel(logging.ERROR)
@@ -92,6 +96,25 @@ def test_ui_ascii(setup_files):
     ui.load_ascii(1, setup_files.ascii, 2, ("col2", "col1"))
 
 
+@requires_fits
+@requires_data
+def test_ui_ascii_noarg(setup_files, clean_astro_ui):
+    """Don't give a dataset id
+
+    It also lets us actually check the results of the load_table call
+    """
+    assert ui.list_data_ids() == []
+    ui.load_ascii(setup_files.ascii)
+    assert ui.list_data_ids() == [1]
+
+    d = ui.get_data()
+    assert d.name.endswith('sim.poisson.1.dat')
+    assert isinstance(d, ui.Data1D)
+    assert len(d.x) == 50
+    assert d.x[0:5] == pytest.approx([0.5, 1, 1.5, 2, 2.5])
+    assert d.y[0:5] == pytest.approx([27, 27, 20, 28, 27])
+
+
 @requires_fits
 @requires_data
 def test_ui_table(setup_files):
@@ -101,6 +124,179 @@ def test_ui_table(setup_files):
     ui.load_table(1, setup_files.fits, 4, ('R', "SUR_BRI", 'SUR_BRI_ERR'),
                   ui.Data1DInt)
 
+
+@requires_fits
+@requires_data
+def test_ui_table_noarg(setup_files, clean_astro_ui):
+    """Don't give a dataset id
+
+    It also lets us actually check the results of the load_table call
+    """
+    assert ui.list_data_ids() == []
+    ui.load_table(setup_files.fits, colkeys=['RMID', 'COUNTS'])
+    assert ui.list_data_ids() == [1]
+
+    d = ui.get_data()
+    assert d.name.endswith('1838_rprofile_rmid.fits')
+    assert isinstance(d, ui.Data1D)
+    assert len(d.x) == 38
+    assert d.x[0:5] == pytest.approx([12.5, 17.5, 22.5, 27.5, 32.5])
+    assert d.y[0:5] == pytest.approx([1529, 2014, 2385, 2158, 2013])
+
+
+def test_dataspace1d_data1dint(clean_astro_ui):
+    """Explicitly test dataspace1d for Data1DInt"""
+
+    assert ui.list_data_ids() == []
+    ui.dataspace1d(20, 30, step=2.5, id='x', dstype=ui.Data1DInt)
+
+    assert ui.list_data_ids() == ['x']
+    assert ui.get_data('x').name == 'dataspace1d'
+
+    grid = ui.get_indep('x')
+    assert len(grid) == 2
+
+    expected = numpy.asarray([20, 22.5, 25, 27.5, 30.0])
+    assert grid[0] == pytest.approx(expected[:-1])
+    assert grid[1] == pytest.approx(expected[1:])
+
+    y = ui.get_dep('x')
+    assert y == pytest.approx(numpy.zeros(4))
+
+
+def test_dataspace1d_datapha(clean_astro_ui):
+    """Explicitly test dataspace1d for DataPHA"""
+
+    assert ui.list_data_ids() == []
+
+    # Note the grid is ignored, other than the number of bins
+    ui.dataspace1d(20, 30, step=2.5, id='x', dstype=ui.DataPHA)
+
+    assert ui.list_data_ids() == ['x']
+    assert ui.get_data('x').name == 'dataspace1d'
+
+    grid = ui.get_indep('x')
+    assert len(grid) == 1
+
+    expected = numpy.asarray([1, 2, 3, 4, 5])
+    assert grid[0] == pytest.approx(expected)
+
+    y = ui.get_dep('x')
+    assert y == pytest.approx(numpy.zeros(5))
+
+    assert ui.get_exposure('x') is None
+    assert ui.get_grouping('x') is None
+    assert ui.get_quality('x') is None
+
+    assert ui.get_data('x').subtracted is False
+
+    with pytest.raises(IdentifierErr):
+        ui.get_bkg('x')
+
+
+def test_dataspace1d_datapha_bkg_nopha(clean_astro_ui):
+    """We need a PHA to create a background dataset"""
+
+    with pytest.raises(IdentifierErr) as exc:
+        ui.dataspace1d(20, 30, step=2.5, id='x', bkg_id=2, dstype=ui.DataPHA)
+
+    assert str(exc.value) == 'data set x has not been set'
+
+
+def test_dataspace1d_datapha_bkg(clean_astro_ui):
+    """Explicitly test dataspace1d for DataPHA (background)"""
+
+    # list_bkg_ids will error out until the dataset exists
+    assert ui.list_data_ids() == []
+
+    # We don't use the grid range or step size since numbins has been
+    # given.
+    ui.dataspace1d(20, 30, step=2.5, numbins=10, id='x', dstype=ui.DataPHA)
+
+    assert ui.list_data_ids() == ['x']
+    assert ui.list_bkg_ids('x') == []
+
+    ui.dataspace1d(20, 30, step=2.5, numbins=10, id='x', bkg_id=2,
+                   dstype=ui.DataPHA)
+
+    assert ui.list_data_ids() == ['x']
+    assert ui.list_bkg_ids('x') == [2]
+
+    assert ui.get_data('x').name == 'dataspace1d'
+
+    # I've explicitly not chosen the default background identifier
+    with pytest.raises(IdentifierErr):
+        ui.get_bkg('x')
+
+    assert ui.get_bkg('x', 2).name == 'bkg_dataspace1d'
+
+    grid = ui.get_indep('x', bkg_id=2)
+    assert len(grid) == 1
+
+    expected = numpy.asarray([1, 2, 3, 4, 5, 6, 7, 8, 9, 10])
+    assert grid[0] == pytest.approx(expected)
+
+    y = ui.get_dep('x', bkg_id=2)
+    assert y == pytest.approx(numpy.zeros(10))
+
+    assert ui.get_exposure('x', bkg_id=2) is None
+    assert ui.get_grouping('x', bkg_id=2) is None
+    assert ui.get_quality('x', bkg_id=2) is None
+
+    assert ui.get_bkg('x', bkg_id=2).subtracted is False
+
+    # check we can subtract the dataset; as the data is all zeros
+    # we don't bother checking the result.
+    #
+    ui.subtract('x')
+
+
+@requires_fits
+@requires_data
+@pytest.mark.parametrize("loader", [ui.load_data, ui.load_pha])
+def test_load_data(loader, make_data_path, clean_astro_ui, caplog):
+    """Ensure that loading a single file to a non-integer id works.
+
+    This is just to make sure that the support for PHA2 files in both
+    load_data and load_pha does not change the single-file case.
+
+    """
+    infile = make_data_path('3c273.pi')
+    bgfile = make_data_path('3c273_bg.pi')
+
+    arf = make_data_path('3c273.arf')
+    rmf = make_data_path('3c273.rmf')
+
+    assert ui.list_data_ids() == []
+
+    with SherpaVerbosity('INFO'):
+        loader('foo', infile)
+
+    assert ui.list_data_ids() == ['foo']
+
+    msg1 = f"systematic errors were not found in file '{infile}'"
+    msg2 = f"statistical errors were found in file '{infile}' \n" + \
+        "but not used; to use them, re-read with use_errors=True"
+    msg3 = f"read ARF file {arf}"
+    msg4 = f"read RMF file {rmf}"
+
+    msg5 = f"systematic errors were not found in file '{bgfile}'"
+    msg6 = f"statistical errors were found in file '{bgfile}' \n" + \
+        "but not used; to use them, re-read with use_errors=True"
+    msg7 = f"read background file {bgfile}"
+
+    assert caplog.record_tuples[0] == ('sherpa.astro.io', logging.WARNING, msg1)
+    assert caplog.record_tuples[1] == ('sherpa.astro.io', logging.INFO, msg2)
+    assert caplog.record_tuples[2] == ('sherpa.astro.io', logging.INFO, msg3)
+    assert caplog.record_tuples[3] == ('sherpa.astro.io', logging.INFO, msg4)
+
+    assert caplog.record_tuples[4] == ('sherpa.astro.io', logging.WARNING, msg5)
+    assert caplog.record_tuples[5] == ('sherpa.astro.io', logging.INFO, msg6)
+    assert caplog.record_tuples[6] == ('sherpa.astro.io', logging.INFO, msg7)
+
+    assert len(caplog.records) == 7
+
+
 # Test table model
 @requires_fits
 @requires_data
@@ -144,6 +340,16 @@ def test_ui_filter_ascii(setup_files):
     ui.load_filter(setup_files.filter_single_int_ascii, ignore=True)
 
 
+@requires_fits
+@requires_data
+def test_ui_filter_ascii_with_id(setup_files):
+    ui.load_filter(1, setup_files.filter_single_int_ascii)
+    assert ui.list_data_ids() == [1]
+
+    f = ui.get_filter()
+    assert f == '2.0000,4.0000,6.0000,8.0000,10.0000:250.0000,751.0000:1000.0000'
+
+
 # Test load_filter
 @requires_fits
 @requires_data
@@ -170,7 +376,7 @@ def test_more_ui_string_model_with_rmf(make_data_path):
     assert isinstance(m, RMFModelPHA)
 
 
-#bug #38
+# bug #38
 @requires_fits
 @requires_data
 @requires_group
@@ -262,6 +468,22 @@ def test_image_12578_set_coord_bad_coord(make_data_path, clean_astro_ui):
     assert okmsg in str(exc.value)
 
 
+@requires_data
+@requires_fits
+def test_image_with_id(make_data_path, clean_astro_ui):
+    """Call load_image with an identifier"""
+
+    img = make_data_path('img.fits')
+
+    assert ui.list_data_ids() == []
+    ui.load_image('ix', img)
+    assert ui.list_data_ids() == ['ix']
+
+    d = ui.get_data('ix')
+    assert isinstance(d, ui.DataIMG)
+    assert d.name.endswith('img.fits')
+
+
 # DJB notes (2020/02/29) that these tests used to not be run,
 # because the lorentz1d model retrned 1 rather than 4. This
 # has now been included in the test so that we can check
@@ -543,17 +765,6 @@ def test_load_table_fits(clean_astro_ui):
     assert data.y == pytest.approx([4, 5, 6])
 
 
-@requires_data
-@requires_fits
-def test_bug_276(make_data_path):
-    ui.load_pha(make_data_path('3c273.pi'))
-    ui.set_model('polynom1d.p1')
-    ui.fit()
-    ui.covar()
-    scal = ui.get_covar_results().parmaxes
-    ui.sample_flux(ui.get_model_component('p1'), 0.5, 1, num=5, correlated=False, scales=scal)
-
-
 @requires_data
 @requires_fits
 def test_bug_316(make_data_path, clean_astro_ui):
@@ -598,6 +809,23 @@ def test_bug_316(make_data_path, clean_astro_ui):
     assert xmax == pytest.approx(7.01)
 
 
+@pytest.mark.parametrize("loadfunc", [ui.load_multi_arfs, ui.load_multi_rmfs])
+@pytest.mark.parametrize("idval", [None, 1, 'xx'])
+def test_load_multi_xxx_invalid_args(loadfunc, idval):
+    """Check we error out if the files and ids do not match."""
+
+    files = ['a', 'b', 'c']
+    rids = [1, 2]
+
+    with pytest.raises(ArgumentErr) as exc:
+        if idval is None:
+            loadfunc(files, rids)
+        else:
+            loadfunc(idval, files, rids)
+
+    assert str(exc.value) == 'A response ID is required for each file'
+
+
 @requires_data
 @requires_fits
 def test_load_multi_arfsrmfs(make_data_path, clean_astro_ui):
@@ -629,6 +857,21 @@ def test_load_multi_arfsrmfs(make_data_path, clean_astro_ui):
     ui.load_multi_rmfs(1, [rmf, rmf], [1, 2])
     ui.load_multi_rmfs(2, [rmf, rmf], [1, 2])
 
+    # Check multiple responses have been loaded
+    #
+    d1 = ui.get_data(1)
+    d2 = ui.get_data(2)
+    assert d1.response_ids == [1, 2]
+    assert d2.response_ids == [1, 2]
+
+    # Unfortunately we load the same response so it's hard
+    # to tell the difference here!
+    #
+    assert ui.get_arf(resp_id=1).name == arf
+    assert ui.get_arf(resp_id=2).name == arf
+    assert ui.get_rmf(2, resp_id=1).name == rmf
+    assert ui.get_rmf(2, resp_id=2).name == rmf
+
     ui.notice(0.5, 7)
     ui.subtract(1)
     ui.subtract(2)
@@ -775,7 +1018,7 @@ def test_list_pileup_ids_multi(clean_astro_ui):
     assert ans == [1, "2"]
 
 
-def check_bad_grouping(exp_xlo,exp_xhi, exp_counts, lo1, hi1, lo2, hi2):
+def check_bad_grouping(exp_xlo, exp_xhi, exp_counts, lo1, hi1, lo2, hi2):
     """Common tests from test_grouped_pha_all_badXXX
 
     Sending in two ranges is a bit excessive but easiest
@@ -872,8 +1115,7 @@ def test_grouped_pha_all_bad_channel(clean_astro_ui):
 @pytest.mark.parametrize("chantype,exp_counts,exp_xlo,exp_xhi,lo1,hi1,lo2,hi2",
                          [("channel", 0.8, 1.0, 6, 0, 7, 2, 6),
                           ("energy", 8.0, 0.1, 0.6, 0.05, 1.0, 0.2, 0.8),
-                          ("wave", 0.03871461, 123.9841874, 20.66403123, 20, 90, 30, 85)
-                         ])
+                          ("wave", 0.03871461, 123.9841874, 20.66403123, 20, 90, 30, 85)])
 def test_grouped_pha_all_bad_response(arf, rmf, chantype, exp_counts, exp_xlo, exp_xhi, lo1, hi1, lo2, hi2, clean_astro_ui):
     """Helpdesk ticket: low-count data had no valid bins after grouping #790
 
@@ -956,3 +1198,85 @@ def test_grouped_pha_all_bad_response_bg_warning(elo, ehi, nbins, bkg_id,
     assert lvl == logging.INFO
     assert msg.startswith('Skipping dataset /')
     assert msg.endswith('/3c273_bg.pi: mask excludes all data')
+
+
+@pytest.mark.parametrize('dtype', [ui.Data1D, ui.Data1DInt, ui.DataPHA,
+                                   ui.Data2D, ui.Data2DInt,
+                                   ui.DataIMG, ui.DataIMGInt])
+def test_unpack_arrays_missing(dtype):
+    """What happens with the wrong number of arguments"""
+
+    with pytest.raises(TypeError):
+        ui.unpack_arrays([1, 2, 5], dtype)
+
+
+def test_unpack_arrays_data1d():
+    """Can we call unpack_arrays? Data1D"""
+
+    ans = ui.unpack_arrays([1, 2, 5], [-2, 3, -1])
+    assert isinstance(ans, ui.Data1D)
+    assert ans.name == ''
+    assert ans.x == pytest.approx([1, 2, 5])
+    assert ans.y == pytest.approx([-2, 3, -1])
+    assert ans.staterror is None
+    assert ans.syserror is None
+
+
+def test_unpack_arrays_data1d_too_many():
+    """What happens with too many columns?
+
+    It just reads in the extra columns, even if "garbage"
+    (such as the negative errors). So technically this is not
+    an error case.
+    """
+
+    ans = ui.unpack_arrays([1, 2, 5], [2, 3, 6], [-2, 3, -1])
+    assert isinstance(ans, ui.Data1D)
+    assert ans.name == ''
+    assert ans.x == pytest.approx([1, 2, 5])
+    assert ans.y == pytest.approx([2, 3, 6])
+    assert ans.staterror == pytest.approx([-2, 3, -1])
+    assert ans.syserror is None
+
+
+def test_unpack_arrays_data1dint():
+    """Can we call unpack_arrays? Data1DInt"""
+
+    ans = ui.unpack_arrays([1, 2, 5], [2, 3, 10], [-2, 3, -1], ui.Data1DInt)
+    assert isinstance(ans, ui.Data1DInt)
+    assert ans.name == ''
+    assert ans.xlo == pytest.approx([1, 2, 5])
+    assert ans.xhi == pytest.approx([2, 3, 10])
+    assert ans.y == pytest.approx([-2, 3, -1])
+    assert ans.staterror is None
+    assert ans.syserror is None
+
+
+def test_unpack_arrays_datapha():
+    """Can we call unpack_arrays? DataPHA"""
+
+    ans = ui.unpack_arrays([1, 2, 3], [2, 0, 4], ui.DataPHA)
+    assert isinstance(ans, ui.DataPHA)
+    assert ans.name == ''
+    assert ans.channel == pytest.approx([1, 2, 3])
+    assert ans.counts == pytest.approx([2, 0, 4])
+    assert ans.staterror is None
+    assert ans.syserror is None
+
+
+def test_unpack_arrays_dataimg():
+    """Can we call unpack_arrays? DataIMG"""
+
+    ivals = numpy.arange(12)
+    y, x = numpy.mgrid[0:3, 0:4]
+    x = x.flatten()
+    y = y.flatten()
+    ans = ui.unpack_arrays(x, y, ivals, (3, 4), ui.DataIMG)
+    assert isinstance(ans, ui.DataIMG)
+    assert ans.name == ''
+    assert ans.x0 == pytest.approx(x)
+    assert ans.x1 == pytest.approx(y)
+    assert ans.y == pytest.approx(ivals)
+    assert ans.shape == pytest.approx(3, 4)
+    assert ans.staterror is None
+    assert ans.syserror is None
