@@ -1,5 +1,5 @@
 #
-#  Copyright (C) 2016, 2017, 2018, 2019, 2020
+#  Copyright (C) 2016, 2017, 2018, 2019, 2020, 2021
 #                Smithsonian Astrophysical Observatory
 #
 #
@@ -19,17 +19,15 @@
 #
 
 import os
-import sys
 import re
 import logging
 
-import pytest
-
 import numpy as np
 from numpy import VisibleDeprecationWarning
-import numpy as np
 
-from sherpa.utils.testing import SherpaTestCase
+import pytest
+
+from sherpa.utils.testing import get_datadir, set_datadir
 
 try:
     from astropy.io.fits.verify import VerifyWarning
@@ -116,7 +114,12 @@ known_warnings = {
             # Matplotlib version 2 warnings (from HTML notebook represention)
             #
             r'np.asscalar\(a\) is deprecated since NumPy v1.16, use a.item\(\) instead',
-            r"Using or importing the ABCs from 'collections' instead of from 'collections.abc' is deprecated since Python 3.3,and in 3.9 it will stop working"
+            r"Using or importing the ABCs from 'collections' instead of from 'collections.abc' is deprecated since Python 3.3,and in 3.9 it will stop working",
+
+            # numpy 1.20 bool/float issue
+            r'`np.bool` is a deprecated alias for the builtin `bool`. .*',
+            r'`np.int` is a deprecated alias for the builtin `int`. .*',
+            r'`np.float` is a deprecated alias for the builtin `float`. .*',
         ],
     UserWarning:
         [
@@ -140,18 +143,11 @@ known_warnings = {
          # See https://github.com/ContinuumIO/anaconda-issues/issues/6678
          r"numpy.dtype size changed, may indicate binary " +
          r"incompatibility. Expected 96, got 88",
-         # I am getting the following from astropy with at least python 2.7 during the conda tests
-         r"numpy.ufunc size changed, may indicate binary ",
+         # See https://github.com/numpy/numpy/pull/432
+         r"numpy.ufunc size changed",
+         # numpy 1.20 shows this in some tests
+         r"numpy.ndarray size changed, may indicate binary "
          ],
-     VisibleDeprecationWarning:
-        [],
-}
-
-# Since Sherpa now requires Python 3.5 at a minumum, the following
-# are always added, but kept as a separate dict and then merged
-# to make it clearer where they came from.
-#
-python3_warnings = {
     ResourceWarning:
         [
             r"unclosed file .*king_kernel.txt.* closefd=True>",
@@ -165,18 +161,15 @@ python3_warnings = {
             # added for sherpa/astro/ui/tests/test_astro_ui_utils_unit.py
             r"unclosed file .*/dev/null.* closefd=True>",
             r"unclosed file .*table.txt.* closefd=True>",
+            # tests in sherpa/astro/ui/tests/test_astro_ui_unit.py
+            # seen in some macOS test runs (e.g. pip, not conda) and python 3.8
+            r"unclosed file .*/data.dat'.* closefd=True>",
+            r"unclosed file .*/model.dat'.* closefd=True>",
+            r"unclosed file .*/resid.out'.* closefd=True>",
         ],
-    RuntimeWarning:
-        [r"invalid value encountered in sqrt",
-         # See https://github.com/ContinuumIO/anaconda-issues/issues/6678
-         r"numpy.dtype size changed, may indicate binary " +
-         r"incompatibility. Expected 96, got 88",
-         # See https://github.com/numpy/numpy/pull/432
-         r"numpy.ufunc size changed"
-         ],
+    VisibleDeprecationWarning:
+        [],
 }
-known_warnings.update(python3_warnings)
-
 
 if have_astropy:
     astropy_warnings = {
@@ -230,13 +223,8 @@ def capture_all_warnings(request, recwarn, pytestconfig):
     pytestconfig injected service for accessing the configuration data
 
     """
-    def known(warning):
-        message = warning.message
-        for known_warning in known_warnings[type(message)]:
-            pattern = re.compile(known_warning)
-            if pattern.match(str(message)):
-                return True
-        return False
+
+    known = check_known_warning
 
     def fin():
         warnings = [w for w in recwarn.list
@@ -263,6 +251,28 @@ def capture_all_warnings(request, recwarn, pytestconfig):
     request.addfinalizer(fin)
 
 
+def check_known_warning(warning):
+    """Return True if this is an "allowed" warning."""
+
+    message = warning.message
+    for known_warning in known_warnings[type(message)]:
+        pattern = re.compile(known_warning)
+        if pattern.match(str(message)):
+            return True
+
+    return False
+
+
+@pytest.fixture
+def is_known_warning():
+    """Returns a function that returns True if this is an "allowed" warning.
+
+    It is not expected that this will see much use.
+    """
+
+    return check_known_warning
+
+
 def pytest_configure(config):
     """
     This configuration hook overrides the default mechanism for test data self-discovery, if the --test-data command line
@@ -277,7 +287,7 @@ def pytest_configure(config):
     try:
         path = config.getoption(TEST_DATA_OPTION)
         if path:
-            SherpaTestCase.datadir = path
+            set_datadir(path)
     except ValueError:  # option not defined from command line, no-op
         pass
 
@@ -296,7 +306,7 @@ def test_data_path():
     path : basestring
         The string with the path to the main test data folder.
     """
-    path = SherpaTestCase.datadir
+    path = get_datadir()
 
     if path is None:
         raise RuntimeError("Test needs the requires_data decorator")
@@ -507,6 +517,10 @@ def reset_seed(request):
 def hide_logging():
     """Set Sherpa's logging to ERROR for the test.
 
+    This code is somehwat redundant with the decorator
+    in utils/logging:SherpaVerbosity and, should this
+    fixture ever need to be redone, it might be worth
+    investigating if the same decorator can be used.
     """
 
     logger = logging.getLogger('sherpa')
