@@ -1,6 +1,6 @@
 #
-#  Copyright (C) 2009, 2015, 2016, 2018, 2019, 2020
-#     Smithsonian Astrophysical Observatory
+#  Copyright (C) 2009, 2015, 2016, 2018, 2019, 2020, 2021
+#  Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -48,11 +48,9 @@ def evaluates_model(func):
     """
     @wraps(func)
     def run(fit, *args, **kwargs):
-        if 'cache' in kwargs:
-            fit.model.startup(kwargs['cache'])
-            kwargs.pop('cache', None)
-        else:
-            fit.model.startup(True)
+
+        cache = kwargs.pop('cache', True)
+        fit.model.startup(cache=cache)
         result = func(fit, *args, **kwargs)
         fit.model.teardown()
         return result
@@ -1059,8 +1057,8 @@ class Fit(NoNewAttributesAfterInit):
 
     def calc_thaw_indices(self):
         self.thaw_indices = \
-            tuple([i for i, par in enumerate(self.model.pars) if not \
-                   par.frozen])
+            tuple([i for i, par in enumerate(self.model.pars)
+                   if not par.frozen])
 
     def __setstate__(self, state):
         self.__dict__.update(state)
