@@ -1,5 +1,5 @@
 #
-#  Copyright (C) 2010, 2016, 2018, 2019, 2020
+#  Copyright (C) 2010, 2016, 2018, 2019, 2020, 2021
 #       Smithsonian Astrophysical Observatory
 #
 #
@@ -116,11 +116,13 @@ class Const(ArithmeticModel):
         ylo = 0
         if numpy.abs(min - 0) > DBL_EPSILON:
             ylo = min / 100.
-            if min < 0: ylo = 0
+            if min < 0:
+                ylo = 0
         yhi = 0
         if numpy.abs(max - 0) > DBL_EPSILON:
             yhi = -10 * max
-            if max > 0: yhi = 100 * max
+            if max > 0:
+                yhi = 100 * max
         param_apply_limits({'val': (max + min) / 2.,
                             'min': ylo, 'max': yhi},
                            self.c0, **kwargs)
@@ -1742,12 +1744,14 @@ class Polynom2D(RegriddableModel2D):
         ylo = 0
         if numpy.abs(ymin - 0) > DBL_EPSILON:
             ylo = -ymin
-            if ymin < 0: ylo = ymin
+            if ymin < 0:
+                ylo = ymin
 
         yhi = 0
         if numpy.abs(ymax - 0) > DBL_EPSILON:
             yhi = -ymax
-            if ymax > 0: yhi = ymax
+            if ymax > 0:
+                yhi = ymax
 
         dydx0 = (ymax - ymin) / (x0max - x0min)
         dydx1 = (ymax - ymin) / (x1max - x1min)
@@ -1755,13 +1759,13 @@ class Polynom2D(RegriddableModel2D):
         dyd2x1 = (ymax - ymin) / ((x1max - x1min) * (x1max - x1min))
         dydx0dx1 = (ymax - ymin) / ((x0max - x0min) * (x1max - x1min))
 
-        c     = {'val': (ymax + ymin) / 2., 'min': ylo, 'max': yhi}
-        cx1   = {'val': 0., 'min': -100 * dydx0, 'max': 100 * dydx0}
-        cy1   = {'val': 0., 'min': -100 * dydx1, 'max': 100 * dydx1}
-        cx2   = {'val': 0., 'min': -100 * dyd2x0, 'max': 100 * dyd2x0}
-        cy2   = {'val': 0., 'min': -100 * dyd2x1, 'max': 100 * dyd2x1}
+        c = {'val': (ymax + ymin) / 2., 'min': ylo, 'max': yhi}
+        cx1 = {'val': 0., 'min': -100 * dydx0, 'max': 100 * dydx0}
+        cy1 = {'val': 0., 'min': -100 * dydx1, 'max': 100 * dydx1}
+        cx2 = {'val': 0., 'min': -100 * dyd2x0, 'max': 100 * dyd2x0}
+        cy2 = {'val': 0., 'min': -100 * dyd2x1, 'max': 100 * dyd2x1}
         cx1y1 = {'val': 0., 'min': -100 * dydx0dx1, 'max': 100 * dydx0dx1}
-        c22   = {'val': 0., 'min': ylo, 'max': yhi }
+        c22 = {'val': 0., 'min': ylo, 'max': yhi}
 
         param_apply_limits(c, self.c, **kwargs)
         param_apply_limits(cx1, self.cx1, **kwargs)
@@ -1779,6 +1783,65 @@ class Polynom2D(RegriddableModel2D):
 
 
 class TableModel(ArithmeticModel):
+    """Tabulated values in this model are simply linearly scaled.
+
+    After initializing this model, the independent and dependent
+    arrays need to be loaded using the ``load`` method before the
+    model can be used (see examples below); interpolation is used if
+    the grids of the data and the table do not match.
+
+    When used with an integrated data set (for example,
+    `Data1DInt`) the independent axis loaded should be the left-edge
+    of the bin, and the dependent axis is the integrated value for that bin.
+
+    This model can also be used as to fit indexed data where the
+    independent axis is not a continuous variable, e.g. the
+    independent variable may hold the index for a number of stars and
+    the dependent variable some measured property for each star. In
+    this case, the independent variable needs to be loaded as ``None``
+    and the length of the dependent array needs to match the length of
+    the data array exactly.
+
+
+    Attributes
+    ----------
+    ampl
+        The linear scaling factor for table values
+
+    See Also
+    --------
+    Const1D
+
+    Examples
+    --------
+    Below is an example for the "indexed" use:
+
+      >>> import numpy as np
+      >>> from sherpa.models.basic import TableModel
+      >>> from sherpa.data import Data1D
+      >>> from sherpa.stats import Chi2
+      >>> from sherpa.optmethods import NelderMead
+      >>> from sherpa.fit import Fit
+      >>> d = Data1D('data', [1,2,3,4,5], [1.2, .4, 2.2, .3, 1.],
+      ...             staterror=[.2, .2, .2, .2, .2])
+      >>> tm = TableModel('tabmodel')
+      >>> tm.load(None, np.array([.6, .2, 1.1, .2, .5]))
+      >>> fit = Fit(d, tm)
+      >>> res = fit.fit()
+
+    This even works for masked data, if the model's ``fold`` method is
+    called first, which informs the model which values are masked.
+
+      >>> d = Data1D('data', [1,2,3,4,5],
+      ...            np.ma.masked_invalid([np.nan, np.nan, 2.2, .3, 1.]),
+      ...            staterror=[.2, .2, .2, .2, .2])
+      >>> tm = TableModel('tabmodel')
+      >>> tm.load(None, np.array([.6, .2, 1.1, .2, .5]))
+      >>> tm.fold(d)
+      >>> fit = Fit(d, tm)
+      >>> res = fit.fit()
+
+    """
 
     def __init__(self, name='tablemodel'):
         # these attributes should remain somewhat private
