@@ -1,6 +1,6 @@
 #
-#  Copyright (C) 2010, 2016, 2017, 2018, 2019, 2020
-#      Smithsonian Astrophysical Observatory
+#  Copyright (C) 2010, 2016, 2017, 2018, 2019, 2020, 2021
+#  Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -18,13 +18,84 @@
 #  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
+"""Allow models to be defined and combined.
 
+A single model is defined by the parameters of the model - represented
+as sherpa.models.model.Parameter instances - and the function that
+takes the parameter values along with an array of grid values. The
+main classes are:
+
+ - Model which is the base class and defines most of the interfaces.
+
+ - ArithmeticConstantModel and ArithmeticFunctionModel for representing
+   a constant value or a function.
+
+ - ArithmeticModel is the main base class for deriving user models since
+   it supports combining models (e.g. by addition or multiplication) and
+   a cache to reduce evaluation time at the expense of memory use.
+
+ - RegriddableModel builds on ArithmeticModel to allow a model to be
+   evaluated on a different grid to that requested: most model classes
+   are derived from the 1D and 2D variants of RegriddableModel.
+
+ - CompositeModel which is used to represent a model expression, that
+   is combined models, such as `m1 * (m2 + m3)`
+
+   - UnaryOpModel for model expressions such as `- m1`.
+
+   - BinaryOpModel for model expressions such as `m1 + m2`.
+
+   - NestedModel for applying one model to another.
+
+ - SimulFitModel for fitting multiple models and datasets.
+
+Model cache
+-----------
+
+The ArithmeticModel class and modelCacher1d decorator provide basic
+support for caching one-dimensional model evaluations - that is, to
+avoid re-calculating the model. The idea is to save the results of the
+latest calls to a model and return the values from the cache,
+hopefully saving time at the expense of using more memory. This is
+most effective when the same model is used with multiple datasets
+which all have the same grid.
+
+The _use_caching attribute of the model is used to determine whether
+the cache is used, but this setting can be over-ridden by the startup
+method, which is automatically called by the fit and est_errors
+methods of a sherpa.fit.Fit object.
+
+The cache_clear and cache_status methods of ArithmeticModel and
+CompositeModel allow you to clear the cache and display to the
+standard output the cache status of each model component.
+
+Example
+-------
+
+The following class implements a simple scale model which has a single
+parameter (`scale`) which defaults to 1. It can be used for both
+non-integrated and integrated datasets of any dimensionality (see
+sherpa.models.basic.Scale1D and sherpa.models.basic.Scale2D)::
+
+    class ScaleND(ArithmeticModel):
+        '''A constant value per element.'''
+
+        def __init__(self, name='scalend'):
+            self.scale = Parameter(name, 'scale', 1)
+            self.integrate = False
+            pars = (self.scale, )
+            ArithmeticModel.__init__(self, name, pars)
+
+        def calc(self, *args, **kwargs):
+            return self.scale.val * np.ones(len(args[0]))
+
+"""
+
+
+import functools
 import logging
-import hashlib
 import warnings
 
-from collections import defaultdict
-
 import numpy
 
 from sherpa.models.regrid import EvaluationSpace1D, ModelDomainRegridder1D, EvaluationSpace2D, ModelDomainRegridder2D
@@ -34,7 +105,18 @@ from sherpa.utils import formatting
 
 from .parameter import Parameter
 
+# What routine do we use for the hash in modelCacher1d?  As we do not
+# need cryptographic security go for a "quick" algorithm, but md5 is
+# not guaranteed to always be present.  There has been no attempt to
+# check the run times of these routines for the expected data sizes
+# they will be used with.
+#
+try:
+    from hashlib import md5 as hashfunc
+except ImportError:
+    from hashlib import sha256 as hashfunc
 
+info = logging.getLogger(__name__).info
 warning = logging.getLogger(__name__).warning
 
 
@@ -45,29 +127,90 @@ __all__ = ('Model', 'CompositeModel', 'SimulFitModel',
 
 
 def boolean_to_byte(boolean_value):
+    """Convert a boolean to a byte value.
+
+    Parameters
+    ----------
+    boolean_value : bool
+        The value to convert. If not a boolean then it is
+        treated as `False`.
+
+    Returns
+    -------
+    val : bytes
+        b'1' if `True` otherwise b'0'.
+    """
+
     bmap = {True: b'1', False: b'0'}
     return bmap.get(boolean_value, b'0')
 
 
 def modelCacher1d(func):
+    """A decorater to cache 1D ArithmeticModel evalutions.
 
+    Apply to the `calc` method of a 1D model to allow the model
+    evaluation to be cached. The decision is based on the
+    `_use_caching` attribute of the cache along with the `integrate`
+    setting, the evaluation grid, and parameter values.
+
+    Example
+    -------
+
+    Allow `MyModel` model evaluations to be cached::
+
+        def MyModel(ArithmeticModel):
+            ...
+            @modelCacher1d
+            def calc(self, *args, **kwargs):
+                ...
+
+    """
+
+    @functools.wraps(func)
     def cache_model(cls, pars, xlo, *args, **kwargs):
         use_caching = cls._use_caching
         cache = cls._cache
+        cache_ctr = cls._cache_ctr
         queue = cls._queue
 
+        # Counts all accesses, even those that do not use the cache.
+        cache_ctr['check'] += 1
+
         digest = ''
         if use_caching:
 
+            # Up until Sherpa 4.12.2 we used the kwargs to define the
+            # integrate setting, with
+            # boolean_to_byte(kwargs.get('integrate', False)) but
+            # unfortunately this is used in code like
+            #
+            #    @modelCacher1d
+            #    def calc(..):
+            #        kwargs['integrate'] = self.integrate
+            #        return somefunc(... **kwargs)
+            #
+            # and the decorator is applied to calc, which is not
+            # called with a integrate kwarg, rather than the call to
+            # somefunc, which was sent an integrate setting.
+            #
+            try:
+                integrate = cls.integrate
+            except AttributeError:
+                # Rely on the integrate kwarg as there's no
+                # model setting.
+                #
+                integrate = kwargs.get('integrate', False)
+
             data = [numpy.array(pars).tobytes(),
-                    boolean_to_byte(kwargs.get('integrate', False)),
+                    boolean_to_byte(integrate),
                     numpy.asarray(xlo).tobytes()]
             if args:
                 data.append(numpy.asarray(args[0]).tobytes())
 
             token = b''.join(data)
-            digest = hashlib.sha256(token).digest()
+            digest = hashfunc(token).digest()
             if digest in cache:
+                cache_ctr['hits'] += 1
                 return cache[digest].copy()
 
         vals = func(cls, pars, xlo, *args, **kwargs)
@@ -81,10 +224,10 @@ def modelCacher1d(func):
             queue.append(digest)
             cache[digest] = vals.copy()
 
+            cache_ctr['misses'] += 1
+
         return vals
 
-    cache_model.__name__ = func.__name__
-    cache_model.__doc__ = func.__doc__
     return cache_model
 
 
@@ -104,13 +247,6 @@ class Model(NoNewAttributesAfterInit):
     pars : sequence of sherpa.parameter.Parameter objects
         The parameters of the model.
 
-    Attributes
-    ----------
-    name : str
-        The name given to the instance.
-    pars : tuple of sherpa.parameter.Parameter objects
-        The parameters of the model instance.
-
     Notes
     -----
     Parameters can be accessed via the ``pars`` attribute, but it is
@@ -121,8 +257,7 @@ class Model(NoNewAttributesAfterInit):
     the variable ``mdl``, then the following can be used to access
     the parameters::
 
-        print("Break frequency = {}".format(mdl.breakfreq))
-
+        print(f"Break frequency = {mdl.breakfreq.val}")
         mdl.norm = 1.2e-3
 
     """
@@ -253,7 +388,7 @@ class Model(NoNewAttributesAfterInit):
 
         See Also
         --------
-        setup
+        startup
         """
         raise NotImplementedError
 
@@ -376,7 +511,7 @@ class Model(NoNewAttributesAfterInit):
         return [p.hard_max for p in self.pars if not p.frozen]
 
     thawedparhardmaxes = property(_get_thawed_par_hardmaxes,
-                                 doc='The hard maximum values for the thawed parameters.')
+                                  doc='The hard maximum values for the thawed parameters.')
 
     def reset(self):
         """Reset the parameter values."""
@@ -432,8 +567,8 @@ class CompositeModel(Model):
             try:
                 self.is_discrete = self.is_discrete or part.is_discrete
             except:
-                warning("Could not determine whether the model is discrete.\n"+
-                        "This probably means that you have restored a session saved with a previous version of Sherpa.\n"+
+                warning("Could not determine whether the model is discrete.\n" +
+                        "This probably means that you have restored a session saved with a previous version of Sherpa.\n" +
                         "Falling back to assuming that the model is continuous.\n")
                 self.is_discrete = False
 
@@ -462,6 +597,34 @@ class CompositeModel(Model):
     def teardown(self):
         pass
 
+    def cache_clear(self):
+        """Clear the cache for each component."""
+        for p in self.parts:
+            try:
+                p.cache_clear()
+            except AttributeError:
+                pass
+
+    def cache_status(self):
+        """Display the cache status of each component.
+
+        Information on the cache - the number of "hits", "misses", and
+        "requests" - is displayed at the INFO logging level.
+
+        Example
+        -------
+
+        >>> mdl.cache_status()
+         xsphabs.gal                size:    5  hits:   715  misses:   158  check=  873
+         powlaw1d.pl                size:    5  hits:   633  misses:   240  check=  873
+
+        """
+        for p in self.parts:
+            try:
+                p.cache_status()
+            except AttributeError:
+                pass
+
 
 class SimulFitModel(CompositeModel):
     """Store multiple models.
@@ -595,16 +758,42 @@ def _make_binop(op, opstr):
 class ArithmeticModel(Model):
     """Support combining model expressions and caching results."""
 
+    cache = 5
+    """The maximum size of the cache."""
+
     def __init__(self, name, pars=()):
         self.integrate = True
 
         # Model caching ability
-        # queue memory of maximum size
-        self.cache = 5
+        self.cache = 5  # repeat the class definition
         self._use_caching = True  # FIXME: reduce number of variables?
+        self.cache_clear()
+        Model.__init__(self, name, pars)
+
+    def cache_clear(self):
+        """Clear the cache."""
+        # It is not obvious what to set the queue length to
         self._queue = ['']
         self._cache = {}
-        Model.__init__(self, name, pars)
+        self._cache_ctr = {'hits': 0, 'misses': 0, 'check': 0}
+
+    def cache_status(self):
+        """Display the cache status.
+
+        Information on the cache - the number of "hits", "misses", and
+        "requests" - is displayed at the INFO logging level.
+
+        Example
+        -------
+
+        >>> pl.cache_status()
+         powlaw1d.pl                size:    5  hits:   633  misses:   240  check=  873
+
+        """
+        c = self._cache_ctr
+        info(f" {self.name:25s}  size: {len(self._queue):4d}  " +
+             f"hits: {c['hits']:5d}  misses: {c['misses']:5d}  " +
+             f"check: {c['check']:5d}")
 
     # Unary operations
     __neg__ = _make_unop(numpy.negative, '-')
@@ -631,6 +820,7 @@ class ArithmeticModel(Model):
 
         if '_cache' not in state:
             self.__dict__['_cache'] = {}
+            self.__dict__['_cache_ctr'] = {'hits': 0, 'misses': 0, 'check': 0}
 
         if 'cache' not in state:
             self.__dict__['cache'] = 5
@@ -639,8 +829,7 @@ class ArithmeticModel(Model):
         return FilterModel(self, filter)
 
     def startup(self, cache=False):
-        self._queue = ['']
-        self._cache = {}
+        self.cache_clear()
         self._use_caching = cache
         if int(self.cache) > 0:
             self._queue = [''] * int(self.cache)
@@ -719,6 +908,9 @@ class UnaryOpModel(CompositeModel, ArithmeticModel):
     arg : Model instance
         The model.
     op : function reference
+        The ufunc to apply to the model values.
+    opstr : str
+        The symbol used to represent the operator.
 
     See Also
     --------
@@ -739,6 +931,7 @@ class UnaryOpModel(CompositeModel, ArithmeticModel):
     def __init__(self, arg, op, opstr):
         self.arg = self.wrapobj(arg)
         self.op = op
+        self.opstr = opstr
         CompositeModel.__init__(self, ('%s(%s)' % (opstr, self.arg.name)),
                                 (self.arg,))
 
@@ -767,6 +960,9 @@ class BinaryOpModel(CompositeModel, RegriddableModel):
     rhs : Model instance
         The right-hand sides of the expression.
     op : function reference
+        The ufunc which combines two array values.
+    opstr : str
+        The symbol used to represent the operator.
 
     See Also
     --------
@@ -789,6 +985,7 @@ class BinaryOpModel(CompositeModel, RegriddableModel):
         self.lhs = self.wrapobj(lhs)
         self.rhs = self.wrapobj(rhs)
         self.op = op
+        self.opstr = opstr
 
         CompositeModel.__init__(self,
                                 ('(%s %s %s)' %
@@ -1054,18 +1251,34 @@ def _wrapobj(obj, wrapper):
 
 # Notebook representation
 #
+def modelcomponents_to_list(model):
+    if hasattr(model, 'parts'):
+        modellist = []
+        for p in model.parts:
+            modellist.extend(modelcomponents_to_list(p))
+        return modellist
+    else:
+        return [model]
+
+
 def html_model(mdl):
     """Construct the HTML to display the model."""
 
     # Note that as this is a specialized table we do not use
     # formatting.html_table but create everything directly.
     #
-    nrows = defaultdict(int)
-    for par in mdl.pars:
-        if par.hidden:
-            continue
-
-        nrows[par.modelname] += 1
+    complist = []
+    nrows = []
+    for comp in modelcomponents_to_list(mdl):
+        this_comp_nrows = 0
+        for par in comp.pars:
+            if par.hidden:
+                continue
+            else:
+                this_comp_nrows +=1
+        if this_comp_nrows > 0:
+            complist.append(comp)
+            nrows.append(this_comp_nrows)
 
     out = '<table class="model">'
     out += '<caption>Expression: {}</caption>'.format(formatting.clean_bracket(mdl.name))
@@ -1076,56 +1289,47 @@ def html_model(mdl):
         out += '<th>{}</th>'.format(col)
 
     out += '</tr></thead><tbody>'
-    seen = set([])
-    mcount = 1
-    for par in mdl.pars:
-        if par.hidden:
-            continue
 
-        def addtd(val):
-            "Use the parameter to convert to HTML"
-            return '<td>{}</td>'.format(par._val_to_html(val))
+    for mcount, (n, comp) in enumerate(zip(nrows, complist)):
+        for i, par in enumerate(comp.pars):
+            style = '' if ((i > 0) or (mcount == 0)) else ' class="block"'
 
-        mname = par.modelname
+            if par.hidden:
+                continue
 
-        style = ''
-        if mname not in seen:
-            if len(seen) > 0:
-                style = ' class="block"'
+            def addtd(val):
+                "Use the parameter to convert to HTML"
+                return '<td>{}</td>'.format(par._val_to_html(val))
 
-        out += '<tr{}>'.format(style)
-        if mname not in seen:
-            cls = "model-" + ("even" if mcount % 2 == 0 else "odd")
-            out += '<th class="{}" scope="rowgroup" '.format(cls)
-            out += 'rowspan={}>{}</th>'.format(nrows[mname], mname)
-            seen.add(mname)
-            mcount += 1
+            out += '<tr{}>'.format(style)
+            if i ==0 :
+                cls = "model-" + ("even" if mcount % 2 == 1 else "odd")
+                out += '<th class="{}" scope="rowgroup" '.format(cls)
+                out += 'rowspan={}>{}</th>'.format(n, comp.name)
 
-        out += '<td>{}</td>'.format(par.name)
+            out += '<td>{}</td>'.format(par.name)
 
-        linked = par.link is not None
-        if linked:
-            out += "<td>linked</td>"
-        else:
-            out += '<td><input disabled type="checkbox"'
-            if not par.frozen:
-                out += ' checked'
-            out += '></input></td>'
+            if par.link is not None:
+                out += "<td>linked</td>"
+            else:
+                out += '<td><input disabled type="checkbox"'
+                if not par.frozen:
+                    out += ' checked'
+                out += '></input></td>'
 
-        out += addtd(par.val)
+            out += addtd(par.val)
 
-        if linked:
-            # 8592 is single left arrow
-            # 8656 is double left arrow
-            #
-            out += '<td colspan=2>&#8656; {}</td>'.format(formatting.clean_bracket(par.link.fullname))
+            if par.link is not None:
+                # 8592 is single left arrow
+                # 8656 is double left arrow
+                #
+                out += '<td colspan=2>&#8656; {}</td>'.format(formatting.clean_bracket(par.link.fullname))
+            else:
+                out += addtd(par.min)
+                out += addtd(par.max)
 
-        else:
-            out += addtd(par.min)
-            out += addtd(par.max)
-
-        out += '<td>{}</td>'.format(par._units_to_html())
-        out += '</tr>'
+            out += '<td>{}</td>'.format(par._units_to_html())
+            out += '</tr>'
 
     out += '</tbody></table>'
 
