@@ -1,5 +1,5 @@
 #
-#  Copyright (C) 2007, 2016, 2017, 2020  Smithsonian Astrophysical Observatory
+#  Copyright (C) 2007, 2016, 2017, 2020, 2021  Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -22,18 +22,27 @@ import logging
 import operator
 import warnings
 
+# Repeat the logic from sherpa/models/model.py
+#
+try:
+    from hashlib import md5 as hashfunc
+except ImportError:
+    from hashlib import sha256 as hashfunc
+
 import numpy
 
 import pytest
 
 from sherpa.utils.err import ModelErr
 from sherpa.models.model import ArithmeticModel, ArithmeticConstantModel, \
-    ArithmeticFunctionModel, BinaryOpModel, FilterModel, NestedModel, UnaryOpModel
+    ArithmeticFunctionModel, BinaryOpModel, FilterModel, Model, NestedModel, \
+    UnaryOpModel, RegridWrappedModel, modelCacher1d
 from sherpa.models.parameter import Parameter, hugeval, tinyval
-from sherpa.models.basic import Sin, Const1D, Box1D
+from sherpa.models.basic import Sin, Const1D, Box1D, Polynom1D
 
 
-def validate_warning(warning_capturer, parameter_name="norm", model_name="ParameterCase", actual_name="Ampl", num=1):
+def validate_warning(warning_capturer, parameter_name="norm",
+                     model_name="ParameterCase", actual_name="Ampl", num=1):
     assert num == len(warning_capturer)
     for warning in warning_capturer:
         assert issubclass(warning.category, DeprecationWarning)
@@ -117,9 +126,9 @@ def setup_parametercase():
 
 def setup_composite():
     out = namedtuple('composite', ['m', 'm2', 's', 'x', 'xx'])
-    out.m  = Const1D('m')
+    out.m = Const1D('m')
     out.m2 = Const1D('m2')
-    out.m.c0  = 2
+    out.m.c0 = 2
     out.m2.c0 = 4
     out.s = Sin('s')
     out.x = 1.0
@@ -545,9 +554,10 @@ def test_functionmodel_check():
 
 
 @pytest.mark.parametrize('model,mtype',
-                         [(ArithmeticConstantModel(23, name='the-23'), ArithmeticConstantModel),
-                          (ArithmeticFunctionModel(numpy.sin), ArithmeticFunctionModel)
-                         ])
+                         [(ArithmeticConstantModel(23, name='the-23'),
+                           ArithmeticConstantModel),
+                          (ArithmeticFunctionModel(numpy.sin),
+                           ArithmeticFunctionModel)])
 def test_unop_arithmeticxxx(model, mtype):
     """Can we apply a function to an Arithmetic*Model object?
 
@@ -571,9 +581,10 @@ def test_unop_arithmeticxxx(model, mtype):
 
 @pytest.mark.parametrize('model,mtype',
                          [(23, ArithmeticConstantModel),
-                          (ArithmeticConstantModel(23, name='the-23'), ArithmeticConstantModel),
-                          (ArithmeticFunctionModel(numpy.sin), ArithmeticFunctionModel)
-                         ])
+                          (ArithmeticConstantModel(23, name='the-23'),
+                           ArithmeticConstantModel),
+                          (ArithmeticFunctionModel(numpy.sin),
+                           ArithmeticFunctionModel)])
 def test_binop_arithmeticxxx(model, mtype):
     """Can we create and combine Arithmetic*Model objects?"""
 
@@ -616,3 +627,498 @@ def test_constant_show(value, name, expected):
 
     m = ArithmeticConstantModel(value, name=name)
     assert m.name == expected
+
+
+def check_cache(mdl, expected, x, xhi=None):
+    """Check the cache contents.
+
+    We assume only one value is being cached at a time. The
+    code matches that in sherpa.models.model.modelCacher1d,
+    so all it does is check we are using this method.
+    """
+
+    cache = mdl._cache
+    assert len(cache) == 1
+
+    pars = [p.val for p in mdl.pars]
+    data = [numpy.asarray(pars).tobytes(),
+            b'1' if mdl.integrate else b'0',
+            x.tobytes()]
+    if xhi is not None:
+        data.append(xhi.tobytes())
+
+    token = b''.join(data)
+    digest = hashfunc(token).digest()
+    assert digest in cache
+    assert cache[digest] == pytest.approx(expected)
+
+
+def test_evaluate_no_cache1d():
+    """Check we can turn off cacheing: 1d"""
+
+    xgrid = numpy.arange(2, 10, 1.5)
+
+    mdl = Polynom1D()
+    mdl.integrate = False
+    mdl._use_caching = False
+    assert len(mdl._cache) == 0
+
+    # Check the default values
+    expected = numpy.ones(6)
+    assert mdl(xgrid) == pytest.approx(expected)
+    assert len(mdl._cache) == 0
+
+    mdl.c0 = 5
+    mdl.c1 = 2
+
+    expected = 5 + 2 * xgrid
+    assert mdl(xgrid) == pytest.approx(expected)
+    assert len(mdl._cache) == 0
+
+
+def test_evaluate_cache1d():
+    """Check we run with cacheing on: 1d"""
+
+    xgrid = numpy.arange(2, 10, 1.5)
+
+    mdl = Polynom1D()
+    mdl.integrate = False
+    mdl._use_caching = True
+    assert len(mdl._cache) == 0
+
+    # Check the default values
+    expected = numpy.ones(6)
+    assert mdl(xgrid) == pytest.approx(expected)
+    check_cache(mdl, expected, xgrid)
+
+    mdl.c0 = 5
+    mdl.c1 = 2
+
+    expected = 5 + 2 * xgrid
+    assert mdl(xgrid) == pytest.approx(expected)
+    check_cache(mdl, expected, xgrid)
+
+
+def test_evaluate_no_cache1dint():
+    """Check we can turn off cacheing: 1dint"""
+
+    xgrid = numpy.arange(2, 10, 1.5)
+    xlo, xhi = xgrid[:-1], xgrid[1:]
+
+    mdl = Polynom1D()
+    mdl._use_caching = False
+    assert len(mdl._cache) == 0
+
+    # Check the default values
+    expected = numpy.ones(5) * 1.5
+    assert mdl(xlo, xhi) == pytest.approx(expected)
+    assert len(mdl._cache) == 0
+
+    mdl.c0 = 5
+    mdl.c1 = 2
+
+    def xval(x):
+        return 5 + 2 * x
+
+    dx = xhi - xlo
+    ylo = xval(xlo)
+    yhi = xval(xhi)
+
+    expected = dx * (yhi + ylo) / 2
+    assert mdl(xlo, xhi) == pytest.approx(expected)
+    assert len(mdl._cache) == 0
+
+
+def test_evaluate_cache1dint():
+    """Check we run with cacheing on: 1dint"""
+
+    xgrid = numpy.arange(2, 10, 1.5)
+    xlo, xhi = xgrid[:-1], xgrid[1:]
+
+    mdl = Polynom1D()
+    mdl._use_caching = True
+    assert len(mdl._cache) == 0
+
+    # Check the default values
+    expected = numpy.ones(5) * 1.5
+    assert mdl(xlo, xhi) == pytest.approx(expected)
+    check_cache(mdl, expected, xlo, xhi)
+
+    mdl.c0 = 5
+    mdl.c1 = 2
+
+    def xval(x):
+        return 5 + 2 * x
+
+    dx = xhi - xlo
+    ylo = xval(xlo)
+    yhi = xval(xhi)
+
+    expected = dx * (yhi + ylo) / 2
+    assert mdl(xlo, xhi) == pytest.approx(expected)
+    check_cache(mdl, expected, xlo, xhi)
+
+
+def test_evaluate_cache_swap():
+    """Check issue #959 when swapping integrate flag caused problems.
+
+    Note that the problem causing #959 is actually tested in
+    test_evaluate_cache1dint but it's nice to have this
+    separate check in case things change.
+    """
+
+    xgrid = numpy.arange(2, 10, 1.5)
+    xlo, xhi = xgrid[:-1], xgrid[1:]
+
+    mdl = Polynom1D()
+    mdl._use_caching = True
+
+    mdl.c0 = 5
+    mdl.c1 = 2
+
+    mdl.integrate = False
+    expected = 5 + 2 * xlo
+
+    y1 = mdl(xlo, xhi)
+    assert y1 == pytest.approx(expected)
+    check_cache(mdl, expected, xlo, xhi)
+
+    mdl.integrate = True
+
+    def xval(x):
+        return 5 + 2 * x
+
+    dx = xhi - xlo
+    ylo = xval(xlo)
+    yhi = xval(xhi)
+
+    expected = dx * (yhi + ylo) / 2
+
+    y2 = mdl(xlo, xhi)
+    assert y2 == pytest.approx(expected)
+    check_cache(mdl, expected, xlo, xhi)
+
+
+def test_evaluate_cache_arithmeticconstant():
+    """Check we run with cacheing: ArihmeticConstant"""
+
+    mdl = ArithmeticConstantModel(2.3)
+    assert not hasattr(mdl, '_use_caching')
+
+
+def test_evaluate_cache_unaryop():
+    """UnaryOp has no cache"""
+
+    mdl = Polynom1D()
+    assert hasattr(mdl, '_use_caching')
+
+    fmdl = -mdl
+    assert isinstance(fmdl, UnaryOpModel)
+    assert not hasattr(fmdl, '_use_caching')
+
+
+def test_evaluate_cache_binaryop():
+    """BinaryOp has no cache"""
+
+    mdl = Polynom1D()
+    assert hasattr(mdl, '_use_caching')
+
+    fmdl = mdl + 2
+    assert isinstance(fmdl, BinaryOpModel)
+    assert not hasattr(fmdl, '_use_caching')
+
+
+def test_evaluate_cache_regrid1d():
+    """How about a regridded model?"""
+
+    mdl = Polynom1D()
+
+    x = numpy.arange(2, 20, 0.5)
+    rmdl = mdl.regrid(x)
+
+    assert isinstance(rmdl, RegridWrappedModel)
+    assert not hasattr(rmdl, '_use_caching')
+
+
+class DoNotUseModel(Model):
+    """This is only used to get a integrate-free model.
+
+    ArithmeticModel sets a integrate field, so this is derived
+    from Model. This requires adding some fields from ArithmeticModel
+    to support use with modelCacher1d.
+    """
+
+    # We need this for modelCacher1d
+    _use_caching = True
+    _cache = {}
+    _cache_ctr = {'hits': 0, 'misses': 0, 'check': 0}
+    _queue = ['']
+
+    @modelCacher1d
+    def calc(self, p, *args, **kwargs):
+        """p is ignored."""
+
+        return numpy.ones(args[0].size)
+
+
+def test_cache_integrate_fall_through_no_integrate():
+    """Try and test the fall-through of the integrate setting.
+
+    This is a bit contrived.
+    """
+
+    mdl = DoNotUseModel('notme')
+    x = numpy.asarray([2, 3, 7, 100])
+    y = mdl(x)
+
+    expected = [1, 1, 1, 1]
+    assert y == pytest.approx(expected)
+
+    # A simplified version of check_cache (as we have no
+    # integrate setting).
+    #
+    cache = mdl._cache
+    assert len(cache) == 1
+
+    pars = []
+    data = [numpy.asarray(pars).tobytes(),
+            b'0', # not integrated
+            x.tobytes()]
+
+    token = b''.join(data)
+    digest = hashfunc(token).digest()
+    assert digest in cache
+    assert cache[digest] == pytest.approx(expected)
+
+
+def test_cache_integrate_fall_through_integrate_true():
+    """See also test_cache_integrate_fall_through_no_integrate."""
+
+    mdl = DoNotUseModel('notme')
+    x = numpy.asarray([2, 3, 7, 100])
+    y = mdl(x, integrate=True)
+
+    expected = [1, 1, 1, 1]
+    assert y == pytest.approx(expected)
+
+    # A simplified version of check_cache (as we have no
+    # integrate setting).
+    #
+    cache = mdl._cache
+    assert len(cache) == 1
+
+    pars = []
+    data = [numpy.asarray(pars).tobytes(),
+            b'1', # integrated
+            x.tobytes()]
+
+    token = b''.join(data)
+    digest = hashfunc(token).digest()
+    assert digest in cache
+    assert cache[digest] == pytest.approx(expected)
+
+
+def test_cache_integrate_fall_through_integrate_false():
+    """See also test_cache_integrate_fall_through_no_integrate."""
+
+    mdl = DoNotUseModel('notme')
+    x = numpy.asarray([2, 3, 7, 100])
+    y = mdl(x, integrate=False)
+
+    expected = [1, 1, 1, 1]
+    assert y == pytest.approx(expected)
+
+    # A simplified version of check_cache (as we have no
+    # integrate setting).
+    #
+    cache = mdl._cache
+    assert len(cache) == 1
+
+    pars = []
+    data = [numpy.asarray(pars).tobytes(),
+            b'0', # not integrated
+            x.tobytes()]
+
+    token = b''.join(data)
+    digest = hashfunc(token).digest()
+    assert digest in cache
+    assert cache[digest] == pytest.approx(expected)
+
+
+def test_cache_status_single(caplog):
+    """Check cache_status for a single model."""
+
+    p = Polynom1D()
+    with caplog.at_level(logging.INFO, logger='sherpa'):
+        p.cache_status()
+
+    assert len(caplog.records) == 1
+    lname, lvl, msg = caplog.record_tuples[0]
+    assert lname == 'sherpa.models.model'
+    assert lvl == logging.INFO
+    toks = msg.split()
+    assert toks[0] == 'polynom1d'
+    assert toks[1] == 'size:'
+    assert toks[2] == '1'
+    assert toks[3] == 'hits:'
+    assert toks[4] == '0'
+    assert toks[5] == 'misses:'
+    assert toks[6] == '0'
+    assert toks[7] == 'check:'
+    assert toks[8] == '0'
+    assert len(toks) == 9
+
+
+def test_cache_status_multiple(caplog):
+    """Check cache_status for a multi-component model.
+
+    Unlike test_cache_syayus_single we also have evaluated the model
+    so we can check that the cache status has changed.
+    """
+
+    # The model expression includes an ArithmeticConstant model (the
+    # term 2) which does not have a cache and so is ignored by
+    # cache_status.
+    #
+    p = Polynom1D()
+    b = Box1D()
+    c = Const1D()
+    mdl = c * (2 * p + b)
+
+    # One model is not cached
+    b._use_caching = False
+
+    mdl([0.1, 0.2, 0.3])
+    mdl([0.1, 0.2, 0.3])
+    mdl([0.1, 0.2, 0.3, 0.4])
+
+    with caplog.at_level(logging.INFO, logger='sherpa'):
+        mdl.cache_status()
+
+    assert len(caplog.records) == 3
+
+    tokens = []
+    for lname, lvl, msg in caplog.record_tuples:
+        assert lname == 'sherpa.models.model'
+        assert lvl == logging.INFO
+        toks = msg.split()
+        assert len(toks) == 9
+        assert toks[1] == 'size:'
+        assert toks[2] == '1'
+        assert toks[3] == 'hits:'
+        assert toks[5] == 'misses:'
+        assert toks[7] == 'check:'
+        assert toks[8] == '3'
+
+        tokens.append(toks)
+
+    toks = tokens[0]
+    assert toks[0] == 'const1d'
+    assert toks[4] == '1'
+    assert toks[6] == '2'
+
+    toks = tokens[1]
+    assert toks[0] == 'polynom1d'
+    assert toks[4] == '1'
+    assert toks[6] == '2'
+
+    toks = tokens[2]
+    assert toks[0] == 'box1d'
+    assert toks[4] == '0'
+    assert toks[6] == '0'
+
+
+def test_cache_clear_single(caplog):
+    """Check cache_clear for a single model."""
+
+    p = Polynom1D()
+
+    # There's no official API for accessing the cache data,
+    # so do it directly.
+    #
+    assert len(p._cache) == 0
+    assert p._cache_ctr['check'] == 0
+    assert p._cache_ctr['hits'] == 0
+    assert p._cache_ctr['misses'] == 0
+
+    p([1, 2, 3])
+    p([1, 2, 3])
+    p([1, 2, 3, 4])
+
+    assert len(p._cache) == 1
+    assert p._cache_ctr['check'] == 3
+    assert p._cache_ctr['hits'] == 1
+    assert p._cache_ctr['misses'] == 2
+
+    p.cache_clear()
+
+    assert len(p._cache) == 0
+    assert p._cache_ctr['check'] == 0
+    assert p._cache_ctr['hits'] == 0
+    assert p._cache_ctr['misses'] == 0
+
+
+def test_cache_clear_multiple(caplog):
+    """Check cache_clear for a combined model."""
+
+    p = Polynom1D()
+    b = Box1D()
+    c = Const1D()
+    mdl = c * (p + 2 * b)
+
+    # Ensure one component doesn't use the cache
+    c._use_caching = False
+
+    # There's no official API for accessing the cache data,
+    # so do it directly.
+    #
+    assert len(p._cache) == 0
+    assert p._cache_ctr['check'] == 0
+    assert p._cache_ctr['hits'] == 0
+    assert p._cache_ctr['misses'] == 0
+
+    assert len(b._cache) == 0
+    assert b._cache_ctr['check'] == 0
+    assert b._cache_ctr['hits'] == 0
+    assert b._cache_ctr['misses'] == 0
+
+    assert len(c._cache) == 0
+    assert c._cache_ctr['check'] == 0
+    assert c._cache_ctr['hits'] == 0
+    assert c._cache_ctr['misses'] == 0
+
+    mdl([1, 2, 3])
+    mdl([1, 2, 3])
+    mdl([1, 2, 3, 4])
+
+    assert len(p._cache) == 1
+    assert p._cache_ctr['check'] == 3
+    assert p._cache_ctr['hits'] == 1
+    assert p._cache_ctr['misses'] == 2
+
+    assert len(b._cache) == 1
+    assert b._cache_ctr['check'] == 3
+    assert b._cache_ctr['hits'] == 1
+    assert b._cache_ctr['misses'] == 2
+
+    assert len(c._cache) == 0
+    assert c._cache_ctr['check'] == 3
+    assert c._cache_ctr['hits'] == 0
+    assert c._cache_ctr['misses'] == 0
+
+    mdl.cache_clear()
+
+    assert len(p._cache) == 0
+    assert p._cache_ctr['check'] == 0
+    assert p._cache_ctr['hits'] == 0
+    assert p._cache_ctr['misses'] == 0
+
+    assert len(b._cache) == 0
+    assert b._cache_ctr['check'] == 0
+    assert b._cache_ctr['hits'] == 0
+    assert b._cache_ctr['misses'] == 0
+
+    assert len(c._cache) == 0
+    assert c._cache_ctr['check'] == 0
+    assert c._cache_ctr['hits'] == 0
+    assert c._cache_ctr['misses'] == 0
