@@ -1,6 +1,6 @@
 #
-#  Copyright (C) 2009, 2015, 2016, 2018, 2019, 2020
-#      Smithsonian Astrophysical Observatory
+#  Copyright (C) 2009, 2015, 2016, 2018, 2019, 2020, 2021
+#  Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,6 @@ A visualization interface to Sherpa
 
 import logging
 import importlib
-import sys
 
 import numpy
 
@@ -83,16 +82,31 @@ backend.init()
 
 plotter = backend
 
-__all__ = ('Plot', 'Contour', 'Point', 'SplitPlot', 'JointPlot',
-           'DataPlot', 'DataHistogramPlot', 'DataContour', 'DelchiPlot', 'ComponentModelPlot',
-           'ModelPlot', 'ModelContour', 'FitPlot', 'FitContour',
-           'ResidPlot', 'ResidContour', 'RatioPlot', 'RatioContour',
-           'IntervalProjection', 'IntervalUncertainty', 'ChisqrPlot',
-           'RegionProjection', 'RegionUncertainty', 'ComponentSourcePlot',
-           'PSFPlot', 'PSFContour', 'begin', 'end', 'exceptions', 'backend',
-           'SourcePlot', 'SourceContour', 'Histogram',
-           'HistogramPlot', 'ModelHistogramPlot', 'SourceHistogramPlot',
-           'plotter')
+__all__ = ('Plot', 'Contour', 'Point', 'Histogram',
+           'HistogramPlot', 'DataHistogramPlot',
+           'ModelHistogramPlot', 'SourceHistogramPlot',
+           'PDFPlot', 'CDFPlot', 'LRHistogram',
+           'SplitPlot', 'JointPlot',
+           'DataPlot', 'TracePlot', 'ScatterPlot',
+           'PSFKernelPlot',
+           'DataContour', 'PSFKernelContour',
+           'ModelPlot', 'ComponentModelPlot',
+           'ComponentModelHistogramPlot', 'ComponentTemplateModelPlot',
+           'SourcePlot', 'ComponentSourcePlot',
+           'ComponentSourceHistogramPlot', 'ComponentTemplateSourcePlot',
+           'PSFPlot',
+           'ModelContour', 'PSFContour', 'SourceContour',
+           'FitPlot',
+           'FitContour',
+           'DelchiPlot', 'ChisqrPlot', 'ResidPlot',
+           'ResidContour',
+           'RatioPlot',
+           'RatioContour',
+           'Confidence1D', 'Confidence2D',
+           'IntervalProjection', 'IntervalUncertainty',
+           'RegionProjection', 'RegionUncertainty',
+           'begin', 'end', 'exceptions', 'backend', 'plotter')
+
 
 _stats_noerr = ('cash', 'cstat', 'leastsq', 'wstat')
 
@@ -166,7 +180,7 @@ def calculate_errors(data, stat, yerrorbars=True):
     # over-ridden once a statistic is given. However, how does this
     # work if the statistic is a Chi2 variant - e.g. Chi2DataVar -
     # but the user has given explicit errors (i.e. they are not to
-    # be calcualted by the "DataVar" part but used as is). Does
+    # be calculated by the "DataVar" part but used as is). Does
     # data.get_yerr handle this for us, or are invalid errors
     # used here? It appears that the correct answers are being
     # returned, but should we only call data.get_yerr if yerr
@@ -298,7 +312,7 @@ class Plot(NoNewAttributesAfterInit):
            The optional plot title and axis labels. These are ignored
            if overplot is set to ``True``.
         overplot : bool, optional
-           If ``True`` then add the data to an exsiting plot, otherwise
+           If ``True`` then add the data to an existing plot, otherwise
            create a new plot.
         clearwindow : bool, optional
            Should the existing plot area be cleared before creating this
@@ -397,7 +411,7 @@ class Point(NoNewAttributesAfterInit):
         x, y
            The coordinates of the plot.
         overplot : bool, optional
-           If ``True`` then add the data to an exsiting plot, otherwise
+           If ``True`` then add the data to an existing plot, otherwise
            create a new plot.
         clearwindow : bool, optional
            Should the existing plot area be cleared before creating this
@@ -452,7 +466,7 @@ class Histogram(NoNewAttributesAfterInit):
            The optional plot title and axis labels. These are ignored
            if overplot is set to ``True``.
         overplot : bool, optional
-           If ``True`` then add the data to an exsiting plot, otherwise
+           If ``True`` then add the data to an existing plot, otherwise
            create a new plot.
         clearwindow : bool, optional
            Should the existing plot area be cleared before creating this
@@ -534,7 +548,7 @@ class HistogramPlot(Histogram):
         """
 
         if self.xlo is None or self.xhi is None:
-            return  None
+            return None
 
         # As we do not (yet) require NumPy arrays, enforce it.
         xlo = numpy.asarray(self.xlo)
@@ -549,7 +563,7 @@ class HistogramPlot(Histogram):
         Parameters
         ----------
         overplot : bool, optional
-           If ``True`` then add the data to an exsiting plot, otherwise
+           If ``True`` then add the data to an existing plot, otherwise
            create a new plot.
         clearwindow : bool, optional
            Should the existing plot area be cleared before creating this
@@ -574,7 +588,7 @@ class HistogramPlot(Histogram):
 # than most (mark by point rather than line).
 #
 def get_data_hist_prefs():
-    """Copy the data prefences to the histogram class"""
+    """Copy the data preferences to the histogram class"""
 
     hprefs = backend.get_model_histo_defaults()
     dprefs = backend.get_data_plot_defaults()
@@ -637,7 +651,7 @@ class DataHistogramPlot(HistogramPlot):
         Parameters
         ----------
         overplot : bool, optional
-           If ``True`` then add the data to an exsiting plot, otherwise
+           If ``True`` then add the data to an existing plot, otherwise
            create a new plot.
         clearwindow : bool, optional
            Should the existing plot area be cleared before creating this
@@ -840,7 +854,7 @@ class CDFPlot(Plot):
         Parameters
         ----------
         overplot : bool, optional
-           If ``True`` then add the data to an exsiting plot, otherwise
+           If ``True`` then add the data to an existing plot, otherwise
            create a new plot.
         clearwindow : bool, optional
            Should the existing plot area be cleared before creating this
@@ -916,7 +930,7 @@ class LRHistogram(HistogramPlot):
         Parameters
         ----------
         overplot : bool, optional
-           If ``True`` then add the data to an exsiting plot, otherwise
+           If ``True`` then add the data to an existing plot, otherwise
            create a new plot.
         clearwindow : bool, optional
            Should the existing plot area be cleared before creating this
@@ -1245,7 +1259,7 @@ class DataPlot(Plot):
         Parameters
         ----------
         overplot : bool, optional
-           If ``True`` then add the data to an exsiting plot, otherwise
+           If ``True`` then add the data to an existing plot, otherwise
            create a new plot.
         clearwindow : bool, optional
            Should the existing plot area be cleared before creating this
@@ -1568,7 +1582,7 @@ class ModelPlot(Plot):
         Parameters
         ----------
         overplot : bool, optional
-           If ``True`` then add the data to an exsiting plot, otherwise
+           If ``True`` then add the data to an existing plot, otherwise
            create a new plot.
         clearwindow : bool, optional
            Should the existing plot area be cleared before creating this
@@ -1618,7 +1632,7 @@ class ComponentTemplateModelPlot(ComponentModelPlot):
 
 
 class SourcePlot(ModelPlot):
-    """Create 1D plots of unconcolved model values.
+    """Create 1D plots of unconvolved model values.
 
     Attributes
     ----------
@@ -1883,7 +1897,7 @@ class FitPlot(Plot):
         Parameters
         ----------
         overplot : bool, optional
-           If ``True`` then add the data to an exsiting plot, otherwise
+           If ``True`` then add the data to an existing plot, otherwise
            create a new plot.
         clearwindow : bool, optional
            Should the existing plot area be cleared before creating this
@@ -2516,6 +2530,23 @@ class Confidence2D(DataContour, Point):
 
     def _region_init(self, fit, par0, par1):
 
+        # Issue #1093 points out that if min or max is a tuple
+        # we can have a problem, as below the code can assign
+        # to an element of one of them. So ensure we have a list
+        # not a tuple.
+        #
+        if self.min is not None:
+            try:
+                self.min = list(self.min)
+            except TypeError:
+                raise ConfidenceErr('badarg', 'Parameter limits', 'a list') from None
+
+        if self.max is not None:
+            try:
+                self.max = list(self.max)
+            except TypeError:
+                raise ConfidenceErr('badarg', 'Parameter limits', 'a list') from None
+
         self.stat = fit.calc_stat()
         self.xlabel = par0.fullname
         self.ylabel = par1.fullname
@@ -2564,10 +2595,6 @@ class Confidence2D(DataContour, Point):
                 if max1 is not None and not numpy.isnan(max1):
                     self.max[1] = par1.val + max1
 
-            # check user limits for errors
-            if numpy.isscalar(self.min) or numpy.isscalar(self.max):
-                raise ConfidenceErr('badarg', 'Parameter limits', 'a list')
-
             for i in [0, 1]:
                 v = (self.max[i] + self.min[i]) / 2.
                 dv = numpy.fabs(v - self.min[i])
