@@ -1,6 +1,6 @@
 #
-#  Copyright (C) 2010, 2015, 2017, 2019, 2020
-#                Smithsonian Astrophysical Observatory
+#  Copyright (C) 2010, 2015, 2017, 2019, 2020, 2021
+#  Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -30,19 +30,19 @@ from sherpa.utils.err import ArgumentErr, NotImplementedErr
 from sherpa.utils import formatting
 
 
-__all__ = ('clear_window','point','plot','histo','contour','set_subplot','init',
-           'get_split_plot_defaults', 'get_plot_defaults', 'begin', 'end',
-           'get_data_plot_defaults', 'get_model_plot_defaults', 'exceptions',
-           'get_fit_plot_defaults', 'get_resid_plot_defaults',
+__all__ = ('clear_window', 'point', 'plot', 'histo', 'contour', 'set_subplot',
+           'init', 'get_split_plot_defaults', 'get_plot_defaults', 'begin',
+           'end', 'get_data_plot_defaults', 'get_model_plot_defaults',
+           'exceptions', 'get_fit_plot_defaults', 'get_resid_plot_defaults',
            'get_ratio_plot_defaults', 'get_contour_defaults',
            'get_data_contour_defaults', 'get_model_contour_defaults',
            'get_fit_contour_defaults', 'get_resid_contour_defaults',
-           'get_ratio_contour_defaults','get_confid_plot_defaults',
+           'get_ratio_contour_defaults', 'get_confid_plot_defaults',
            'get_confid_contour_defaults', 'set_window_redraw', 'set_jointplot',
            'get_model_histo_defaults', 'get_histo_defaults',
-           'get_component_plot_defaults','get_component_histo_defaults',
-           'vline', 'hline', 'get_scatter_plot_defaults', 'get_cdf_plot_defaults',
-           'get_latex_for_string', 'name')
+           'get_component_plot_defaults', 'get_component_histo_defaults',
+           'vline', 'hline', 'get_scatter_plot_defaults',
+           'get_cdf_plot_defaults', 'get_latex_for_string', 'name')
 
 
 # Name of the backend
@@ -129,7 +129,7 @@ def setup_plot(axes, title, xlabel, ylabel, xlog=False, ylog=False):
 
 
 def find_zorder(axes):
-    """Try to come up with a good zoder value
+    """Try to come up with a good zorder value
 
     Parameters
     ----------
@@ -167,7 +167,7 @@ def find_zorder(axes):
     2.1 from plot/errorbar. This should work for the case of
       plot something
       overplot something
-    but may fall appart as soon as users add their own features
+    but may fall apart as soon as users add their own features
     to the visualization, but that may be acceptable.
     """
 
@@ -176,7 +176,7 @@ def find_zorder(axes):
     objects = axes.lines + axes.collections
 
     # could do a list comprehension, but want to catch
-    # AtributeErrors
+    # AttributeErrors
     zs = []
     for o in objects:
         try:
@@ -227,7 +227,7 @@ def histo(xlo, xhi, y, yerr=None, title=None, xlabel=None, ylabel=None,
     """Draw histogram data.
 
     The histogram is drawn as horizontal lines connecting the
-    start and end points of each bin, with vetical lines connecting
+    start and end points of each bin, with vertical lines connecting
     consecutive bins. Non-consecutive bins are drawn with a
     (Nan,NaN) between them so no line is drawn connecting them.
 
@@ -316,7 +316,6 @@ def histo(xlo, xhi, y, yerr=None, title=None, xlabel=None, ylabel=None,
                   color=color,
                   alpha=alpha,
                   linestyle='',
-                  drawstyle=drawstyle,
                   marker=marker,
                   markersize=markersize,
                   markerfacecolor=markerfacecolor,
@@ -446,7 +445,6 @@ def plot(x, y, yerr=None, xerr=None, title=None, xlabel=None, ylabel=None,
         objs = axes.errorbar(x, y, yerr, xerr,
                              color=color,
                              linestyle=linestyle,
-                             drawstyle=drawstyle,
                              marker=marker,
                              markersize=markersize,
                              markerfacecolor=markerfacecolor,
@@ -507,7 +505,7 @@ def contour(x0, x1, y, levels=None, title=None, xlabel=None, ylabel=None,
 
     x0 = numpy.unique(x0)
     x1 = numpy.unique(x1)
-    y  = numpy.asarray(y)
+    y = numpy.asarray(y)
 
     if x0.size * x1.size != y.size:
         raise NotImplementedErr('contourgrids')
@@ -590,7 +588,7 @@ def set_jointplot(row, col, nrows, ncols, create=True,
         fig.subplots_adjust(hspace=0.05)
 
         # Change all but the bottom row. By setting sharex
-        # this is probably un-needed as get_xticklabels is empty.
+        # this is probably unneeded as get_xticklabels is empty.
         #
         if axes.ndim == 2:
             axes = axes[:-1, :].flatten()
