@@ -1,5 +1,5 @@
 #
-#  Copyright (C) 2019 Smithsonian Astrophysical Observatory
+#  Copyright (C) 2019, 2020, 2021 Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -16,15 +16,19 @@
 #  with this program; if not, write to the Free Software Foundation, Inc.,
 #  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
+
 import numpy
+
 import pytest
 
-from sherpa.data import Data, Data1D, DataSimulFit, Data1DInt, Data2D, Data2DInt, BaseData
+from sherpa.data import Data, Data1D, DataSimulFit, Data1DInt, Data2D, \
+    Data2DInt, BaseData
 from sherpa.models import Polynom1D, Polynom2D
 from sherpa.utils.err import NotImplementedErr, DataErr
 from sherpa.ui.utils import Session
 from sherpa.astro.ui.utils import Session as AstroSession
 
+
 NAME = "data_test"
 X_ARRAY = numpy.arange(0, 10, 1)
 Y_ARRAY = numpy.arange(100, 110, 1)
@@ -620,7 +624,8 @@ def test_data_simul_fit_eval_model_to_fit(data_simul_fit):
     data_simul_fit.datasets[0].mask = X_ARRAY <= X_THRESHOLD
     data_simul_fit.datasets[1].mask = X_ARRAY <= X_THRESHOLD
     evaluated_data = data_simul_fit.eval_model_to_fit((model, model))
-    expected_data = numpy.concatenate((MULTIPLIER * X_ARRAY[:X_THRESHOLD+1], MULTIPLIER **2 * X_ARRAY[:X_THRESHOLD+1]))
+    expected_data = numpy.concatenate((MULTIPLIER * X_ARRAY[:X_THRESHOLD+1],
+                                       MULTIPLIER**2 * X_ARRAY[:X_THRESHOLD+1]))
     numpy.testing.assert_array_equal(evaluated_data, expected_data)
 
 
@@ -922,8 +927,7 @@ def test_data2_int_get_indep_notice(data):
     expected = [(X0_2D - 0.5)[test_filter_0],
                 (X0_2D + 0.5)[test_filter_0],
                 (X1_2D - 0.5)[test_filter_1],
-                (X1_2D + 0.5)[test_filter_1],
-               ]
+                (X1_2D + 0.5)[test_filter_1]]
     actual = data.get_indep(filter=True)
     numpy.testing.assert_array_equal(actual[0], expected[0])
     numpy.testing.assert_array_equal(actual[1], expected[1])
@@ -1028,6 +1032,7 @@ def test_data_indep_masked_numpyarray(arrpos, Dataclass):
         data = Dataclass(*args)
     assert len(data.get_dep(filter=True)) == len(args[POS_Y_ARRAY[Dataclass]])
 
+
 @pytest.mark.parametrize('arrpos', [POS_STATERR_ARRAY, POS_SYSERR_ARRAY])
 @pytest.mark.parametrize("Dataclass", ALL_DATA_CLASSES)
 def test_data_err_masked_numpyarray(arrpos, Dataclass):
@@ -1051,8 +1056,8 @@ def test_data_deperr_masked_numpyarray(arrpos, Dataclass):
     args[i] = numpy.ma.array(args[i], mask=mask)
     args[j] = numpy.ma.array(args[j], mask=mask)
     data = Dataclass(*args)
-    assert len(data.get_dep(filter=True)) == (~mask).sum()    
-    
+    assert len(data.get_dep(filter=True)) == (~mask).sum()
+
 
 @pytest.mark.parametrize("Dataclass", REALLY_ALL_DATA_CLASSES)
 def test_data_dep_masked_numpyarray(Dataclass):
@@ -1071,7 +1076,7 @@ def test_data_dep_masked_numpyarray_nomask(Dataclass):
     args = list(INSTANCE_ARGS[Dataclass])
     posy = POS_Y_ARRAY[Dataclass]
     # By default, numpy creates a masked array with no mask set
-    args[posy] = numpy.ma.array(args[posy]) 
+    args[posy] = numpy.ma.array(args[posy])
     data = Dataclass(*args)
     # Sherpa's way of saying "mask is not set"
     assert data.mask is True
@@ -1081,6 +1086,7 @@ def test_data_dep_masked_numpyarray_nomask(Dataclass):
 @pytest.mark.parametrize("Dataclass", ALL_DATA_CLASSES)
 def test_data_indep_anyobj_with_mask(Dataclass):
     args = list(INSTANCE_ARGS[Dataclass])
+
     class DummyMask(list):
         mask = 'whatisthis'
     args[1] = DummyMask(args[1])
@@ -1094,6 +1100,7 @@ def test_data_indep_anyobj_with_mask(Dataclass):
 def test_data_dep_any_obj_with_mask(Dataclass):
     args = list(INSTANCE_ARGS[Dataclass])
     posy = POS_Y_ARRAY[Dataclass]
+
     class DummyMask(list):
         mask = 'whatisthis'
     args[posy] = DummyMask(args[posy])
@@ -1158,15 +1165,15 @@ def test_data_dep_masked_numpyarray_nomask_ui(data_for_load_arrays, Session):
 @pytest.mark.parametrize('Session', [Session, AstroSession])
 def test_regression_346(Session):
     session = Session()
-    x = numpy.arange(-5, 5.1) 
+    x = numpy.arange(-5, 5.1)
     old_y = x*x + 23.2
-    y = numpy.ma.masked_array(old_y,mask=old_y<35) 
-    e = numpy.ones(x.size) 
-    session.load_arrays("mydata", x, y, e) 
+    y = numpy.ma.masked_array(old_y, mask=old_y < 35)
+    e = numpy.ones(x.size)
+    session.load_arrays("mydata", x, y, e)
     filtered_y = session. get_dep("mydata", filter=True)
     assert numpy.allclose(filtered_y, [48.2, 39.2, 39.2, 48.2])
 
-    
+
 def test_manual_setting_mask():
     d = Data1D(name='test', x=[1, 2, 3], y=[0, 0, 0])
     d.mask = True
@@ -1185,13 +1192,11 @@ def test_manual_setting_mask():
 
     d.mask = [True, False, True]
     assert len(d.get_dep(filter=True)) == 2
-    arr = numpy.ma.array([3,4,5])
-    d.mask = arr.mask  # aka numpy.ma.nomask, but used in a more
-                       # natural way
+    arr = numpy.ma.array([3, 4, 5])
+    # aka numpy.ma.nomask, but used in a more natural way
+    d.mask = arr.mask
     assert len(d.get_dep(filter=True)) == 3
 
     with pytest.raises(DataErr) as e:
         d.mask = None
     assert 'True, False, or a mask array' in str(e.value)
-
-        
