@@ -1,5 +1,6 @@
 #
-#  Copyright (C) 2019, 2020  Smithsonian Astrophysical Observatory
+#  Copyright (C) 2019, 2020, 2021
+#  Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -40,12 +41,12 @@ from sherpa.models import basic
 from sherpa.plot import CDFPlot, DataPlot, FitPlot, ModelPlot, \
     PDFPlot, PSFPlot, PSFKernelPlot, ScatterPlot, TracePlot,\
     DataContour, ModelContour, SourceContour, ResidContour, \
-    RatioContour, FitContour, PSFContour, LRHistogram, \
+    RatioContour, FitContour, LRHistogram, \
     ModelHistogramPlot, ResidPlot, RatioPlot, DelchiPlot, ChisqrPlot, \
     DataHistogramPlot
 
 from sherpa.stats import Chi2Gehrels
-from sherpa.utils.err import ArgumentErr, ArgumentTypeErr
+from sherpa.utils.err import ArgumentErr, ArgumentTypeErr, IdentifierErr
 from sherpa.utils.testing import requires_plotting, requires_pylab
 
 
@@ -146,9 +147,6 @@ def test_plot_prefs_xxx(session, ptype, arg):
     defaults to 'False'; b) each plot type has this setting; c) we
     do not need to check all settings.
 
-    Some tests fail due to missing plot preferences when there's
-    no plotting backend (e.g. missing 'xlog' settings), so skip
-    these tests in this case.
     """
 
     s = session()
@@ -1113,7 +1111,7 @@ def test_plot_single(session):
 
     ax = fig.axes[0]
 
-    assert ax.get_geometry() == (1, 1, 1)
+    assert ax.get_subplotspec().get_geometry() == (1, 1, 0, 0)
     assert ax.get_title() == ''
     assert ax.xaxis.get_label().get_text() == 'x'
     assert ax.yaxis.get_label().get_text() == 'y'
@@ -1127,7 +1125,7 @@ def test_plot_single(session):
 
     ax = fig.axes[0]
 
-    assert ax.get_geometry() == (1, 1, 1)
+    assert ax.get_subplotspec().get_geometry() == (1, 1, 0, 0)
     assert ax.get_title() == 'Model'
     assert ax.xaxis.get_label().get_text() == 'x'
     assert ax.yaxis.get_label().get_text() == 'y'
@@ -1177,9 +1175,10 @@ def test_plot_multiple(session):
                                                  'Ratio of Data to Model'],
                                                 ['y', 'y', 'y', 'y',
                                                  'Data / Model']),
-                                    1):
+                                            1):
 
-        assert ax.get_geometry() == (2, 3, i)
+        w = i - 1
+        assert ax.get_subplotspec().get_geometry() == (2, 3, w, w)
         assert ax.get_title() == title
         assert ax.xaxis.get_label().get_text() == 'x'
         assert ax.yaxis.get_label().get_text() == ylabel
@@ -1226,7 +1225,7 @@ def test_contour_single(session):
 
     ax = fig.axes[0]
 
-    assert ax.get_geometry() == (1, 1, 1)
+    assert ax.get_subplotspec().get_geometry() == (1, 1, 0, 0)
     assert ax.get_title() == ''
     assert ax.xaxis.get_label().get_text() == 'x0'
     assert ax.yaxis.get_label().get_text() == 'x1'
@@ -1240,7 +1239,7 @@ def test_contour_single(session):
 
     ax = fig.axes[0]
 
-    assert ax.get_geometry() == (1, 1, 1)
+    assert ax.get_subplotspec().get_geometry() == (1, 1, 0, 0)
     assert ax.get_title() == 'Model'
     assert ax.xaxis.get_label().get_text() == 'x0'
     assert ax.yaxis.get_label().get_text() == 'x1'
@@ -1287,7 +1286,8 @@ def test_contour_multiple(session):
                                          'Ratio of Data to Model']),
                                     1):
 
-        assert ax.get_geometry() == (2, 3, i)
+        w = i - 1
+        assert ax.get_subplotspec().get_geometry() == (2, 3, w, w)
         assert ax.get_title() == title
         assert ax.xaxis.get_label().get_text() == 'x0'
         assert ax.yaxis.get_label().get_text() == 'x1'
@@ -1343,7 +1343,8 @@ def test_contour_xxx(plotfunc, title, pcls, session):
                                             ['', 'Residuals']),
                                         1):
 
-            assert ax.get_geometry() == (2, 1, i)
+            w = i - 1
+            assert ax.get_subplotspec().get_geometry() == (2, 1, w, w)
             assert ax.get_title() == title
             assert ax.xaxis.get_label().get_text() == 'x0'
             assert ax.yaxis.get_label().get_text() == 'x1'
@@ -1352,7 +1353,7 @@ def test_contour_xxx(plotfunc, title, pcls, session):
         assert len(fig.axes) == 1
 
         ax = fig.axes[0]
-        assert ax.get_geometry() == (1, 1, 1)
+        assert ax.get_subplotspec().get_geometry() == (1, 1, 0, 0)
         assert ax.get_title() == title
         assert ax.xaxis.get_label().get_text() == 'x0'
         assert ax.yaxis.get_label().get_text() == 'x1'
@@ -1572,7 +1573,7 @@ def test_get_cdf_plot_empty(session):
 
     s = session()
     p = s.get_cdf_plot()
-    assert isinstance (p, CDFPlot)
+    assert isinstance(p, CDFPlot)
     for f in ["points", "x", "y", "median", "lower", "upper", "xlabel", "ylabel", "title"]:
         assert getattr(p, f) is None
 
@@ -1638,7 +1639,7 @@ def test_plot_cdf(session):
     s.plot_cdf(x)
 
     p = s.get_cdf_plot()
-    assert isinstance (p, CDFPlot)
+    assert isinstance(p, CDFPlot)
 
     xsort = np.sort(x)
     assert p.points == pytest.approx(x)
@@ -1662,7 +1663,7 @@ def test_plot_cdf_labels_noname(session):
     s.plot_cdf(x, xlabel='a b')
 
     p = s.get_cdf_plot()
-    assert isinstance (p, CDFPlot)
+    assert isinstance(p, CDFPlot)
 
     assert p.xlabel == 'a b'
     assert p.ylabel == 'p(<=a b)'
@@ -1679,7 +1680,7 @@ def test_plot_cdf_labels(session):
     s.plot_cdf(x, xlabel='a b', name='b a')
 
     p = s.get_cdf_plot()
-    assert isinstance (p, CDFPlot)
+    assert isinstance(p, CDFPlot)
 
     assert p.xlabel == 'a b'
     assert p.ylabel == 'p(<=a b)'
@@ -1693,7 +1694,7 @@ def test_show_cdf_plot_empty(session):
     s = session()
 
     p = s.get_cdf_plot()
-    assert isinstance (p, CDFPlot)
+    assert isinstance(p, CDFPlot)
 
     toks = str(p).split('\n')
     assert len(toks) == 10
@@ -1724,7 +1725,7 @@ def test_show_cdf_plot(session, old_numpy_printing):
     s.plot_cdf(x)
 
     p = s.get_cdf_plot()
-    assert isinstance (p, CDFPlot)
+    assert isinstance(p, CDFPlot)
 
     toks = str(p).split('\n')
     assert len(toks) == 10
@@ -1745,7 +1746,7 @@ def test_get_pdf_plot_empty(session):
 
     s = session()
     p = s.get_pdf_plot()
-    assert isinstance (p, PDFPlot)
+    assert isinstance(p, PDFPlot)
     for f in ["points", "xlo", "xhi", "y", "xlabel", "ylabel", "title"]:
         assert getattr(p, f) is None
 
@@ -1760,7 +1761,7 @@ def test_plot_pdf(session):
     s.plot_pdf(x)
 
     p = s.get_pdf_plot()
-    assert isinstance (p, PDFPlot)
+    assert isinstance(p, PDFPlot)
 
     xgrid = np.arange(2, 8.5, 0.5)
 
@@ -1855,7 +1856,7 @@ def test_plot_pdf_labels_noname(session):
     s.plot_pdf(x, xlabel='x^2 x', bins=4)
 
     p = s.get_pdf_plot()
-    assert isinstance (p, PDFPlot)
+    assert isinstance(p, PDFPlot)
 
     assert p.xlo.size == 4
     assert p.xhi.size == 4
@@ -1876,7 +1877,7 @@ def test_plot_pdf_labels(session):
     s.plot_pdf(x, xlabel='x^2 x', name='no name', bins=4)
 
     p = s.get_pdf_plot()
-    assert isinstance (p, PDFPlot)
+    assert isinstance(p, PDFPlot)
 
     assert p.xlo.size == 4
     assert p.xhi.size == 4
@@ -1894,7 +1895,7 @@ def test_show_pdf_plot_empty(session):
     s = session()
 
     p = s.get_pdf_plot()
-    assert isinstance (p, PDFPlot)
+    assert isinstance(p, PDFPlot)
 
     toks = str(p).split('\n')
     assert len(toks) == 8
@@ -1923,7 +1924,7 @@ def test_show_pdf_plot(session, old_numpy_printing):
     s.plot_pdf(x, bins=3, normed=False)
 
     p = s.get_pdf_plot()
-    assert isinstance (p, PDFPlot)
+    assert isinstance(p, PDFPlot)
 
     toks = str(p).split('\n')
     assert len(toks) == 8
@@ -1960,6 +1961,46 @@ def test_data_plot_recalc(session):
     assert p.y == pytest.approx([10, 12, 14])
 
 
+@pytest.mark.parametrize("session", [BaseSession, AstroSession])
+@pytest.mark.parametrize("ptype,extraargs",
+                         [('model', []), ('model_component', ['mdl']),
+                          ('source', []), ('source_component', ['mdl'])])
+def test_xxx_plot_nodata(ptype, extraargs, session):
+    """Basic testing of get_xxx_plot when there's no data"""
+
+    s = session()
+    s._add_model_types(basic)
+
+    mdl = s.create_model_component('polynom1d', 'mdl')
+    mdl.c0 = 10
+    mdl.c1 = 1
+    s.set_source(mdl)
+
+    func = getattr(s, 'get_{}_plot'.format(ptype))
+    retval = func(*extraargs, recalc=False)
+    assert retval.y is None
+
+
+@pytest.mark.parametrize("session", [BaseSession, AstroSession])
+@pytest.mark.parametrize("ptype,extraargs",
+                         [('model', []), ('model_component', ['mdl']),
+                          ('source', []), ('source_component', ['mdl'])])
+def test_xxx_plot_nodata_recalc(ptype, extraargs, session):
+    """Basic testing of get_xxx_plot when there's no data and recalc=True"""
+
+    s = session()
+    s._add_model_types(basic)
+
+    mdl = s.create_model_component('polynom1d', 'mdl')
+    mdl.c0 = 10
+    mdl.c1 = 1
+    s.set_source(mdl)
+
+    func = getattr(s, 'get_{}_plot'.format(ptype))
+    with pytest.raises(IdentifierErr):
+        func(*extraargs)
+
+
 @pytest.mark.parametrize("session", [BaseSession, AstroSession])
 @pytest.mark.parametrize("ptype,extraargs",
                          [('model', []), ('model_component', ['mdl']),
@@ -2046,7 +2087,6 @@ def test_xxx_plot_recalc(ptype, pclass, y1, y2, session):
     assert p.y == pytest.approx(y2)
 
 
-
 @pytest.mark.parametrize("session", [BaseSession, AstroSession])
 def test_fit_plot_recalc(session):
     """Basic testing of get_fit_plot(recalc=False)"""
@@ -2326,7 +2366,6 @@ def test_pylab_plot_trace_empty_replot(session):
         s.plot_trace(y, replot=True)
 
 
-
 @requires_pylab
 @pytest.mark.parametrize("session", [BaseSession, AstroSession])
 def test_pylab_plot_trace(session):
@@ -2539,7 +2578,7 @@ def test_ratio_contour_recalc(session):
 
     ygood = np.isfinite(p.y)
     assert ygood.sum() == ygood.size - 1
-    assert ygood[40] == False
+    assert not ygood[40]
 
 
 @pytest.mark.parametrize("session", [BaseSession, AstroSession])
@@ -2587,7 +2626,7 @@ def test_resid_contour_recalc(session):
 
     ygood = np.isfinite(p.y)
     assert ygood.sum() == ygood.size - 1
-    assert ygood[40] == False
+    assert not ygood[40]
 
 
 @pytest.mark.parametrize("session", [BaseSession, AstroSession])
@@ -2631,7 +2670,7 @@ def test_fit_contour_recalc(session):
     assert pd.x1 == pytest.approx(x1)
     assert pm.x0 == pytest.approx(x0)
     assert pm.x1 == pytest.approx(x1)
-    assert pd.y== pytest.approx(y)
+    assert pd.y == pytest.approx(y)
     # just check the model isn't flat
     assert pm.y.min() < pm.y.max()
 
@@ -2645,7 +2684,7 @@ def test_fit_contour_recalc(session):
     assert pd.x1 == pytest.approx(nx1)
     assert pm.x0 == pytest.approx(nx0)
     assert pm.x1 == pytest.approx(nx1)
-    assert pd.y== pytest.approx(ny)
+    assert pd.y == pytest.approx(ny)
     # just check the model is flat
     assert pm.y.min() == pm.y.max()
 
@@ -2653,7 +2692,7 @@ def test_fit_contour_recalc(session):
 @requires_pylab
 @pytest.mark.parametrize("ptype",
                          ["resid", "ratio", "delchi"])
-def test_plot_fit_xxx_pylab(ptype):
+def test_plot_fit_xxx_pylab(ptype, clean_ui):
     """Just ensure we can create a plot_fit_xxx call."""
 
     from matplotlib import pyplot as plt
@@ -2719,3 +2758,62 @@ def test_plot_fit_xxx_overplot_pylab(ptype, caplog):
         l0 = axes[idx].lines
         assert l0[0].get_xydata() == pytest.approx(l0[2].get_xydata())
         assert l0[1].get_xydata() == pytest.approx(l0[3].get_xydata())
+
+
+@requires_pylab
+@pytest.mark.parametrize("idval", [None, "bob"])
+def test_plot_fit_resid_handles_data_log(idval, clean_ui):
+    """Check that log handling is correct: data=log
+
+    See also test_plot_fit_resid_handles_resid_log.
+
+    I thought we had tests of this, but apparently not.
+    """
+
+    from matplotlib import pyplot as plt
+
+    setup_example(idval)
+    ui.set_xlog('data')
+    ui.plot_fit_resid(idval)
+
+    fig = plt.gcf()
+    axes = fig.axes
+    assert len(axes) == 2
+    assert axes[0].xaxis.get_label().get_text() == ''
+
+    assert axes[0].xaxis.get_scale() == 'log'
+    assert axes[0].yaxis.get_scale() == 'linear'
+
+    assert axes[1].xaxis.get_scale() == 'log'
+    assert axes[1].yaxis.get_scale() == 'linear'
+
+
+@requires_pylab
+@pytest.mark.parametrize("idval", [None, "bob"])
+def test_plot_fit_resid_handles_resid_log(idval, clean_ui):
+    """Check that log handling is correct: resid=log
+
+    We need to decide whether we want the residual setting to override
+    the linear display of the fit plot here. At present the code is
+    that if resid has xlog set then both will be drawn logged (since
+    the X axis is shared via a sharex=True argument to plt.subplots)
+    but we may decide this should change.
+
+    """
+
+    from matplotlib import pyplot as plt
+
+    setup_example(idval)
+    ui.set_xlog('resid')
+    ui.plot_fit_resid(idval)
+
+    fig = plt.gcf()
+    axes = fig.axes
+    assert len(axes) == 2
+    assert axes[0].xaxis.get_label().get_text() == ''
+
+    assert axes[0].xaxis.get_scale() == 'log'
+    assert axes[0].yaxis.get_scale() == 'linear'
+
+    assert axes[1].xaxis.get_scale() == 'log'
+    assert axes[1].yaxis.get_scale() == 'linear'
